def test_send_order_limit_price(trade_channel, instrument_id, exchange_id, direction, open_close, price, volume):
    """
    测试限价发单
    :param trade_channel:
    :param instrument_id:
    :param exchange_id:
    :param direction:
    :param open_close:
    :param price:
    :param volume:
    """
    if trade_channel.broker_id != '9999':
        print('非SimNow账户不允许测试发单......')
        return None
    return trade_channel.send_order_limit_price(instrument_id, exchange_id, direction, open_close, price, volume)


def test_send_order_any_price(trade_channel, instrument_id, exchange_id, direction, open_close, volume):
    """
    测试市价发单
    :param trade_channel:
    :param instrument_id:
    :param exchange_id:
    :param direction:
    :param open_close:
    :param volume:
    """
    if trade_channel.broker_id != '9999':
        print('非SimNow账户不允许测试发单......')
        return None
    return trade_channel.send_order_any_price(instrument_id, exchange_id, direction, open_close, volume)


def test_cancel_order(trade_channel, order):
    """
    测试撤单
    :param trade_channel:
    :param order:
    """
    trade_channel.cancel_order(order)


def test_query_instrument(trade_channel, instrument_id=None):
    """
    测试查询合约
    :param trade_channel:
    :param instrument_id:
    """
    lst_future_info = trade_channel.query_instrument(instrument_id=instrument_id)
    for future_info in lst_future_info:
        print(future_info.to_string())


def test_query_asset(trade_channel):
    """
    测试查询账户资金
    :param trade_channel:
    """
    asset = trade_channel.query_asset()
    print(asset.to_string())


def test_query_orders(trade_channel):
    """
    测试查询委托
    :param trade_channel:
    """
    lst_data = trade_channel.query_orders()
    for data in lst_data:
        print(data.to_string())


def test_query_trades(trade_channel):
    """
    测试查询交易
    :param trade_channel:
    """
    lst_data = trade_channel.query_trades()
    for data in lst_data:
        print(data.to_string())


def test_query_positions(trade_channel):
    """
    测试查询持仓
    :param trade_channel:
    """
    lst_data = trade_channel.query_positions()
    for data in lst_data:
        print(data.to_string())


def test_query_position_details(trade_channel):
    """
    测试查询持仓明细
    :param trade_channel:
    """
    lst_data = trade_channel.query_position_details()
    lst_data.sort(key=lambda x: x.open_time, reverse=True)
    for data in lst_data:
        print(data.to_string())


def test_query_settlement_info(trade_channel, trading_date):
    """
    测试查询结算单
    :param trading_date:
    :param trade_channel:
    """
    settlement_info = trade_channel.query_settlement_info(trading_date)
    print(settlement_info)


def test_query_asset_by_trading_date(trade_channel, trading_date):
    """
    测试查询账户信息
    :param trade_channel:
    :param trading_date:
    """
    asset = trade_channel.query_asset_by_trading_date(trading_date)
    print(asset.to_string())



from SapphireQuant.Core.Enum import EnumProductClass
from SapphireQuant.Core.Enum.EnumLifePhase import EnumLifePhase
from SapphireQuant.Core.Enum.EnumMarket import EnumMarket
from SapphireQuant.Core.Tick import Tick


class Instrument:
    """
    合约
    """
    def __init__(self):
        self._id = ""
        self._can_sel_today_pos = True
        self._market = EnumMarket.期货
        self._exchange_id = "SHFE"
        self._name = ""
        self._last_tick = Tick()
        self._last_price = 0.0
        self._pre_close = 0.0
        self._pre_settlement_price = 0.0
        self._up_limit = 0.0
        self._drop_limit = 0.0
        self._volume_multiple = 1
        self._price_tick = 0.01
        self._life_phase = EnumLifePhase.上市
        self._up_limit_ratio = 0.0
        self._drop_limit_ratio = 0.0
        self._lot_size = 0
        self._product_id = ""
        self.product_class = EnumProductClass.future

        self._tick_event = None

    @property
    def id(self):
        """

        :return:
        """
        return self._id

    @id.setter
    def id(self, value):
        self._id = value

    @property
    def can_sel_today_pos(self):
        """

        :return:
        """
        return self._can_sel_today_pos

    @can_sel_today_pos.setter
    def can_sel_today_pos(self, value):
        self._can_sel_today_pos = value

    @property
    def market(self):
        """

        :return:
        """
        return self._market

    @market.setter
    def market(self, value):
        self._market = value

    @property
    def exchange_id(self):
        """

        :return:
        """
        return self._exchange_id

    @exchange_id.setter
    def exchange_id(self, value):
        self._exchange_id = value

    @property
    def name(self):
        """

        :return:
        """
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    @property
    def last_tick(self):
        """

        :return:
        """
        return self._last_tick

    @last_tick.setter
    def last_tick(self, value):
        self._last_tick = value

    @property
    def last_price(self):
        """

        :return:
        """
        return self._last_price

    @last_price.setter
    def last_price(self, value):
        self._last_price = value

    @property
    def pre_close(self):
        """

        :return:
        """
        return self._pre_close

    @pre_close.setter
    def pre_close(self, value):
        self._pre_close = value

    @property
    def pre_settlement_price(self):
        """

        :return:
        """
        return self._pre_settlement_price

    @pre_settlement_price.setter
    def pre_settlement_price(self, value):
        self._pre_settlement_price = value

    @property
    def up_limit(self):
        """

        :return:
        """
        return self._up_limit

    @up_limit.setter
    def up_limit(self, value):
        self._up_limit = value

    @property
    def rop_limit(self):
        """

        :return:
        """
        return self._drop_limit

    @rop_limit.setter
    def rop_limit(self, value):
        self._drop_limit = value

    @property
    def volume_multiple(self):
        """

        :return:
        """
        return self._volume_multiple

    @volume_multiple.setter
    def volume_multiple(self, value):
        self._volume_multiple = value

    @property
    def price_tick(self):
        """

        :return:
        """
        return self._price_tick

    @price_tick.setter
    def price_tick(self, value):
        self._price_tick = value

    @property
    def life_phase(self):
        """

        :return:
        """
        return self._life_phase

    @life_phase.setter
    def life_phase(self, value):
        self._life_phase = value

    @property
    def up_limit_ratio(self):
        """

        :return:
        """
        return self._up_limit_ratio

    @up_limit_ratio.setter
    def up_limit_ratio(self, value):
        self._up_limit_ratio = value

    @property
    def drop_limit_ratio(self):
        """

        :return:
        """
        return self._drop_limit_ratio

    @drop_limit_ratio.setter
    def drop_limit_ratio(self, value):
        self._drop_limit_ratio = value

    @property
    def lot_size(self):
        """

        :return:
        """
        return self._lot_size

    @lot_size.setter
    def lot_size(self, value):
        self._lot_size = value

    @property
    def product_id(self):
        """

        :return:
        """
        return self._product_id

    @product_id.setter
    def product_id(self, value):
        self._product_id = value

    def on_tick(self, tick):
        """

        :param tick:
        """
        if tick is not None:
            tick.ExchangeId = self._exchange_id
            self._last_tick = tick
            if self._tick_event is not None:
                try:
                    self._tick_event(tick)
                except Exception as e:
                    print(str(e))

    def to_string(self):
        """

        :return:
        """
        string_builder = ""
        string_builder += ("[" + self._id + "]")
        string_builder += self._name + ","
        string_builder += self._exchange_id + ","
        string_builder += "PriceTick=" + str(self._price_tick) + ","
        string_builder += "VolumeMultiple=" + str(self._volume_multiple) + ","
        string_builder += "昨收=" + str(self._pre_close) + ","
        return string_builder



import os
import shutil
import compileall

versions = ".cpython-35"
py_cache = "__pycache__"
build_dir = "build"
root = os.path.abspath('.')
print("正在编译文件...请等待...")
compileall.compile_dir(root)  # 编译
build = os.path.join(root, build_dir)
if os.path.exists(build):
    print("正在移除build目录...请等待...")
    shutil.rmtree(build)
print("正在复制文件...请等待...")
shutil.copytree(root, build)
for root, dirs, files in os.walk(build):  # 开始遍历文件
    # root 表示当前正在访问的文件夹路径
    # dirs 表示该文件夹下的子目录名list
    # files 表示该文件夹下的文件list
    # 遍历文件
    for file_name in files:
        src = os.path.join(root, file_name)
        if file_name.endswith(".py"):
            os.remove(src)
        elif file_name.endswith(".pyc"):
            upper_dir = root.replace(py_cache, "")
            dst = os.path.join(upper_dir, file_name.replace(versions, ""))
            shutil.copy(src, dst)
for root, dirs, files in os.walk(build):  # 移除缓存文件夹
    print("正在移除缓存文件...请等待...")
    if root.endswith(py_cache):
        shutil.rmtree(root)
print("打包完成...请查看目录：" + str(build))



import datetime

from SapphireQuant.Config.TradingDay import TradingDayHelper


class FutureTradingTime:
    """
    期货自然日交易时间
    """

    def __init__(self, instrument_id, trading_date, lst_time_slice):
        self.instrument_id = instrument_id
        self.trading_date = trading_date

        self.if_night_flag = False
        self.night_open = TradingDayHelper.get_natural_date_time(trading_date, datetime.timedelta(hours=21, minutes=0))
        self.night_close = TradingDayHelper.get_natural_date_time(trading_date, datetime.timedelta(hours=2, minutes=30))

        self.day_am_open = trading_date + datetime.timedelta(hours=9, minutes=0)
        self.day_am_rest_begin = trading_date + datetime.timedelta(hours=10, minutes=30)
        self.day_am_rest_end = trading_date + datetime.timedelta(hours=10, minutes=30)
        self.day_am_close = trading_date + datetime.timedelta(hours=11, minutes=30)
        self.day_pm_open = trading_date + datetime.timedelta(hours=13, minutes=0)
        self.day_pm_close = trading_date + datetime.timedelta(hours=15, minutes=0)

        if lst_time_slice is not None and len(lst_time_slice) > 0:
            for time_slice in lst_time_slice:
                if datetime.timedelta(hours=20) < time_slice.begin_time < datetime.timedelta(hours=23):
                    self.if_night_flag = True
                    self.night_open = TradingDayHelper.get_natural_date_time(trading_date, time_slice.begin_time)
                    self.night_close = TradingDayHelper.get_natural_date_time(trading_date, time_slice.end_time)

                if datetime.timedelta(hours=5) < time_slice.begin_time < datetime.timedelta(hours=10):
                    self.day_am_open = trading_date + time_slice.begin_time

                if datetime.timedelta(hours=10, minutes=0) < time_slice.end_time < datetime.timedelta(hours=10, minutes=20):
                    self.day_am_rest_begin = trading_date + time_slice.end_time

                if datetime.timedelta(hours=10, minutes=20) < time_slice.begin_time < datetime.timedelta(hours=10, minutes=40):
                    self.day_am_rest_end = trading_date + time_slice.begin_time

                if datetime.timedelta(hours=11, minutes=20) < time_slice.end_time < datetime.timedelta(hours=11, minutes=40):
                    self.day_am_close = trading_date + time_slice.end_time

                if datetime.timedelta(hours=12, minutes=20) < time_slice.begin_time < datetime.timedelta(hours=13, minutes=40):
                    self.day_pm_open = trading_date + time_slice.begin_time

                if datetime.timedelta(hours=14, minutes=50) < time_slice.end_time < datetime.timedelta(hours=16, minutes=0):
                    self.day_pm_close = trading_date + time_slice.end_time

        self.auction_begin = self.day_am_open + datetime.timedelta(minutes=-4)
        self.auction_end = self.day_am_open + datetime.timedelta(minutes=-1)
        self.auction_match_time = self.auction_end

    def to_string(self):
        """

        :return:
        """
        msg = '[{0}],是否有夜盘:{1}\n'.format(self.instrument_id, self.if_night_flag)
        msg += '夜盘：[0-1]\n'.format(self.night_open.stftime('%Y%m%d %H:%M:%S'), self.night_close.stftime('%Y%m%d %H:%M:%S'))
        if self.day_am_rest_begin == self.day_am_rest_end:
            msg += '上午：[0-1]\n'.format(self.day_am_open.stftime('%Y%m%d %H:%M:%S'), self.day_am_close.stftime('%Y%m%d %H:%M:%S'))
        else:
            msg += '上午：[0-1],[2-3]\n'.format(self.day_am_open.stftime('%Y%m%d %H:%M:%S'),
                                             self.day_am_rest_begin.stftime('%Y%m%d %H:%M:%S'),
                                             self.day_am_rest_end.stftime('%Y%m%d %H:%M:%S'),
                                             self.day_am_close.stftime('%Y%m%d %H:%M:%S'))
        msg += '下午：[0-1]'.format(self.day_pm_open.stftime('%Y%m%d %H:%M:%S'), self.day_pm_close.stftime('%Y%m%d %H:%M:%S'))
        return msg



from enum import Enum, unique


@unique
class EnumPositionDirection(Enum):
    多头 = 0
    空头 = 1
    净持仓 = 2
    备兑 = 3

    def chinese_name(self):
        """

        :return:
        """
        if self == EnumPositionDirection.多头:
            return '多头'
        elif self == EnumPositionDirection.空头:
            return '空头'
        elif self == EnumPositionDirection.净持仓:
            return '净持仓'
        else:
            return '备兑'



from enum import Enum, unique


@unique
class EnumOrderType(Enum):
    normal = 48
    derive_from_quote = 49
    derivce_from_combination = 50
    combination = 51
    conditional_order = 52
    swap = 53
    derive_from_block_trade = 54
    derive_from_EFP_trade = 55



import datetime
import os
import os.path

from SapphireQuant import BarSeries
from SapphireQuant.Core.Bar import Bar
from SapphireQuant.Core.CSharpUtils import CSharpUtils
from SapphireQuant.Core.Enum.EnumBarType import EnumBarType
from SapphireQuant.DataProcessing.BinaryWriter import BinaryWriter
from SapphireQuant.DataProcessing.BinaryReader import BinaryReader
from SapphireQuant.DataProcessing.FileHeader import FileHeader


class BinaryMinBarStream(object):
    """
    分钟读写流
    """
    BarLength = 88
    SimpleDayBarCountThreshold = 18

    def __init__(self, market, instrument_id, file_path):
        self._file_stream = None
        self._file_header = FileHeader()
        self._file_extension = ".min"
        self._instrument_id = ""
        self._file_path = ""

        self._file_header.file_version = "MQ 1.0"
        self._file_header.period = 1
        self._file_header.market = market
        self._file_header.bar_type = EnumBarType.minute
        self._file_path = file_path

        if file_path.strip() == '':
            raise Exception("Invalid Config File Path!")

        extension = os.path.splitext(file_path)[1].lower()
        if extension != self._file_extension:
            raise Exception("Invalid File Extension!")

        create_new = not os.path.exists(file_path)
        self._instrument_id = instrument_id.split('.')[0]

        if create_new:
            self._file_stream = open(file_path, 'w')
            self._file_stream.close()

        self._file_stream = open(file_path, 'rb+')

        if not create_new:
            if os.path.getsize(file_path) == 0:
                raise Exception("Invalid File Length!")
            if self._file_header is None:
                self._file_header = FileHeader()

    def _read_bar(self, reader):
        """
        读取Bar
        :param reader:
        :return:
        """
        bar = Bar()
        bar.begin_time = CSharpUtils.convert_c_sharp_ticks_to_py_date_time(reader.read_int64())
        bar.end_time = CSharpUtils.convert_c_sharp_ticks_to_py_date_time(reader.read_int64())
        bar.trading_date = CSharpUtils.convert_c_sharp_ticks_to_py_date_time(reader.read_int64())
        bar.open = reader.read_double()
        bar.close = reader.read_double()
        bar.high = reader.read_double()
        bar.low = reader.read_double()
        bar.pre_close = reader.read_double()
        bar.volume = reader.read_double()
        bar.turnover = reader.read_double()
        bar.open_interest = reader.read_double()
        bar.instrument_id = self._instrument_id
        # bar.IsCompleted = True

        return bar

    def read_all(self, bar_series):
        """

        :param bar_series:
        :return:
        """
        if bar_series is None:
            raise Exception("BarSeries Is Empty.")

        reader = BinaryReader(self._file_stream)
        self._file_header.read(reader)
        begin_date = self._file_header.begin_trading_day
        end_date = self._file_header.end_trading_day

        return self.read_trading_days(bar_series, begin_date, end_date)

    def read_trading_day(self, bar_series, trading_day):
        """
        按照交易日读取分钟线数据
        :param bar_series:分钟线数据
        :param trading_day:交易日
        :return:
        """
        if bar_series is None:
            raise Exception("BarSeries Is Empty.")

        if not isinstance(trading_day, datetime.date):
            raise Exception("Trading Day Param Error")

        read_count = 0
        trading_day = datetime.datetime(trading_day.year, trading_day.month, trading_day.day)
        reader = BinaryReader(self._file_stream)
        self._file_header.read(reader)
        if (trading_day < self._file_header.begin_trading_day) or (trading_day > self._file_header.end_trading_day):
            return False

        data = self._file_header.trading_day_indices.try_get(trading_day)
        found = data[0]
        start_offset = data[1]
        count = data[2]
        if found:
            reader.stream.seek(start_offset, 0)
            while count > 0:
                bar = self._read_bar(reader)
                bar_series.append(bar)
                count -= 1
                read_count += 1

        return read_count > 0

    def read_trading_days(self, bar_series, begin_trading_day, end_trading_day):
        """
        按照交易日区间读取分钟线数据
        :param bar_series:分钟线数据
        :param begin_trading_day:开始交易日
        :param end_trading_day:结束交易日
        :return:
        """
        if bar_series is None:
            raise Exception("BarSeries Is Empty.")

        if not isinstance(begin_trading_day, datetime.date):
            raise Exception("BeginTradingDay Param Error.")

        if not isinstance(end_trading_day, datetime.date):
            raise Exception("EndTradingDay Param Error.")

        begin_trading_day = datetime.datetime(begin_trading_day.year, begin_trading_day.month, begin_trading_day.day)
        end_trading_day = datetime.datetime(end_trading_day.year, end_trading_day.month, end_trading_day.day)

        if begin_trading_day > end_trading_day:
            raise Exception("EndTradingDay Must Be Larger Than BeginTradingDay")

        if begin_trading_day == end_trading_day:
            return self.read_trading_day(bar_series, begin_trading_day)

        read_count = 0
        reader = BinaryReader(self._file_stream)
        self._file_header.read(reader)
        if (end_trading_day < self._file_header.begin_trading_day) or (
                begin_trading_day > self._file_header.end_trading_day):
            return False

        if begin_trading_day < self._file_header.begin_trading_day:
            begin_trading_day = self._file_header.begin_trading_day

        if end_trading_day > self._file_header.end_trading_day:
            end_trading_day = self._file_header.end_trading_day

        data = self._file_header.trading_day_indices.get_trading_day_offsets(begin_trading_day, end_trading_day)
        found = data[0]
        start_offset = data[1]
        end_offset = data[2]
        start_count = data[3]
        end_count = data[4]
        if found:
            end_offset += self.BarLength * end_count
            reader.stream.seek(start_offset, 0)
            while reader.stream.tell() < end_offset:
                bar = self._read_bar(reader)
                bar_series.append(bar)
                read_count += 1

        return read_count > 0

    def _update_file_header(self, bar_series, start, length):
        start_bar = bar_series[start]
        end_bar = bar_series[start + length - 1]
        if self._file_header.begin_time == datetime.datetime(2999, 1, 1) or start_bar.begin_time < self._file_header.begin_time:
            self._file_header.begin_time = start_bar.begin_time
            self._file_header.mark_as_dirty(True)

        if self._file_header.end_time == datetime.datetime(2999, 1, 1) or start_bar.end_time > self._file_header.end_time:
            self._file_header.end_time = end_bar.end_time
            self._file_header.mark_as_dirty(True)
        self._file_header.bar_count += length

    def _write_interval(self, bar_series: BarSeries, start: int, length: int, offset: int):
        if bar_series is None:
            raise Exception("Argument Null Exception:bar_series")
        if length < 0:
            raise Exception("Argument Null Exception:length")
        if start < 0 or (start + length > len(bar_series)):
            raise Exception("Argument Null Exception:start")

        self._update_file_header(bar_series, start, length)
        writer = BinaryWriter(self._file_stream)
        writer.stream.seek(offset, 0)
        self._write_bars(writer, bar_series, 0, length)
        self._file_header.write(writer)

    def _write_bars(self, writer, bar_series, start, length):
        for i in range(start, length, 1):
            bar = bar_series[i]
            self._write_bar(writer, bar)

    def _write_bar(self, writer, bar):
        trading_date = datetime.datetime(bar.trading_date.year, bar.trading_date.month, bar.trading_date.day)
        if self._file_header.begin_trading_day == datetime.datetime.max or trading_date < self._file_header.begin_trading_day:
            self._file_header.begin_trading_day = trading_date
            self._file_header.mark_as_dirty(True)
        if self._file_header.end_trading_day == datetime.datetime.max or trading_date > self._file_header.end_trading_day:
            self._file_header.end_trading_day = trading_date
            self._file_header.mark_as_dirty(True)
        offset = writer.stream.tell()
        self._file_header.natural_day_indices.add(bar.begin_time, offset)
        self._file_header.trading_day_indices.add(bar.trading_date, offset)
        writer.write_int64(CSharpUtils.get_c_sharp_ticks(bar.begin_time))
        writer.write_int64(CSharpUtils.get_c_sharp_ticks(bar.end_time))
        writer.write_int64(CSharpUtils.get_c_sharp_ticks(bar.trading_date))
        writer.write_double(bar.open)
        writer.write_double(bar.close)
        writer.write_double(bar.high)
        writer.write_double(bar.low)
        writer.write_double(bar.pre_close)
        writer.write_double(bar.volume)
        writer.write_double(bar.turnover)
        writer.write_double(bar.open_interest)

    def write(self, bar_series):
        self.write_bar_series(bar_series, 0, len(bar_series))
        self._file_stream.flush()

    def write_bar_series(self, bar_series: BarSeries, start: int, length: int):
        self._write_interval(bar_series, start, length, FileHeader.FileHeaderTotalSize)
        self._file_stream.flush()



import abc
import datetime
import os
import time

import pandas as pd

from SapphireQuant.Config.TradingDay import TradingDayHelper
from SapphireQuant.Core.Enum import EnumBuySell, EnumOpenClose, EnumPositionDirection, EnumBarType, EnumMarket, EnumPositionType
from SapphireQuant.Core.Futures import FutureAsset, FutureOrder, FutureTrade
from SapphireQuant.DataStorage.Cache import FutureData, FutureCache
from SapphireQuant.DataStorage.Cache.FutureCache import EnumLiquidationType
from SapphireQuant.Lib import FileToolBox
from SapphireQuant.Simulate import ReplayData
from SapphireQuant.Strategy.StrategyToolBox import StrategyToolBox


class OldStrategy:
    """
    策略基类 不带Function,不带撮合机的
    """

    def __init__(self, **kwargs):
        self.kwargs = kwargs
        if 'mongodb_client' in kwargs.keys():
            self.mongodb_client = kwargs['mongodb_client']
        else:
            self.mongodb_client = None
        if 'record_dir' in kwargs.keys():
            self.record_dir = kwargs['record_dir']
        else:
            self.record_dir = None
        self.product_code = kwargs['product_code']
        self.strategy_id = kwargs['strategy_id']
        self.tick_now = None
        self.last_bar = None
        self.last_tick = None
        self.future_data = None
        self.drive_code = kwargs['drive_code']
        self.subscribe_futures = kwargs['subscribe_futures']
        self.drive_bar_interval = kwargs['drive_bar_interval']
        self.drive_bar_type = kwargs['drive_bar_type']
        self.start_simulate_date = kwargs['start_simulate_date']
        self.end_simulate_date = kwargs['end_simulate_date']
        self.trading_date = self.start_simulate_date
        self.data_handler = kwargs['data_handler']
        self.fund_balance = kwargs['fund_balance']
        self.instrument_manager = self.data_handler.instrument_manager
        self.transaction_rate = kwargs['transaction_rate']
        self.long_margin_rate = kwargs['long_margin_rate']
        self.short_margin_rate = kwargs['short_margin_rate']

        self.liquidation_type = EnumLiquidationType.收盘价
        if 'liquidation_type' in kwargs.keys():
            self.liquidation_type = kwargs['liquidation_type']

        self.future_cache = FutureCache(self.transaction_rate, self.long_margin_rate, self.short_margin_rate,
                                        self.data_handler.instrument_manager, self.data_handler, liquidation_type=self.liquidation_type)

        self.all_futures = {}

        self.drive_replay_data = ReplayData(self.drive_code)
        self.other_replay_data_map = {}
        self.last_bar_map = {}
        self.subscribe(self.subscribe_futures)

        self.all_assets = []
        self.all_orders = []
        self.all_trades = []
        self.all_positions = []
        self.all_position_details = []

        self.df_asset = None
        self.lst_asset = []

        self.df_order = None
        self.lst_order = []

        self.df_trade = None
        self.lst_trade = []

        self.df_position = None
        self.lst_position = []

        self.df_position_detail = None
        self.lst_position_detail = []

        self.df_summary = None
        self.df_summary_by_year = None
        self.df_summary_filter = None
        self.df_param = None
        self.run_seconds = 0.0
        self.indicators = []

        self.trading_day_num = None
        self.total_trade_lots = None
        self.volume_per_day = None
        self.yield_rate = None
        self.total_yield_rate = None
        self.annual_yield_rate = None
        self.profit_per_hand = None
        self.max_retrace_rate = None
        self.max_retrace = None
        self.retrace_ratio = None
        self.sharpe_ratio = None
        self.max_daily_loss = None
        self.continuous_innovation_high_days = None
        self.max_loss_days = None
        self.lst_summary = None

        self.is_allow = True  # 主要用来管理已跑过的策略自动不执行
        self.order_sys_id = -1
        self.trade_sys_id = -1

    def add_indicator(self, indicator):
        """
        添加指标
        :param indicator:
        """
        indicator.calculate()
        self.indicators.append(indicator)

    def subscribe(self, instrument_ids):
        """
        订阅品种
        :param instrument_ids:合约代码ID序列
        """
        for instrument_id in instrument_ids:
            if instrument_id not in self.all_futures.keys():
                self.all_futures[instrument_id] = self.instrument_manager[instrument_id]
                if instrument_id != self.drive_code:
                    self.other_replay_data_map[instrument_id] = ReplayData(instrument_id)
                self.last_bar_map[instrument_id] = None

    def start_simulate(self):
        """
        开始回测
        """
        t0 = time.time()
        if self.drive_bar_type == EnumBarType.day:
            # 初始化数据
            for instrument_id in self.subscribe_futures:
                bar_series = self.data_handler.load_bar_series_by_date(
                    EnumMarket.期货, instrument_id, self.drive_bar_interval, self.drive_bar_type, self.start_simulate_date, self.end_simulate_date)
                if instrument_id == self.drive_code:
                    for bar in bar_series:
                        self.drive_replay_data.add_data(bar.trading_date, [bar])
                else:
                    for bar in bar_series:
                        self.other_replay_data_map[instrument_id].add_data(bar.trading_date, [bar])

            self.on_start()
            lst_trading_days = TradingDayHelper.get_trading_days(self.start_simulate_date, self.end_simulate_date)
            for trading_date in lst_trading_days:
                self.trading_date = trading_date
                self.order_sys_id = 1
                self.trade_sys_id = 1
                self.data_handler.init(trading_date)
                self.__replay_bar(trading_date)
        elif (self.drive_bar_type == EnumBarType.minute) or (self.drive_bar_type == EnumBarType.hour):
            self.on_start()
            lst_trading_days = TradingDayHelper.get_trading_days(self.start_simulate_date, self.end_simulate_date)
            for trading_date in lst_trading_days:
                self.trading_date = trading_date
                self.order_sys_id = 1
                self.trade_sys_id = 1
                self.data_handler.init(trading_date)
                for instrument_id in self.subscribe_futures:
                    bar_series = self.data_handler.load_bar_series_by_date(
                        EnumMarket.期货, instrument_id, self.drive_bar_interval, self.drive_bar_type, trading_date, trading_date)
                    if instrument_id == self.drive_code:
                        self.drive_replay_data.add_data(trading_date, bar_series)
                    else:
                        self.other_replay_data_map[instrument_id].add_data(trading_date, bar_series)

                self.__replay_bar(trading_date)
        elif self.drive_bar_type == EnumBarType.tick:
            self.on_start()
            lst_trading_days = TradingDayHelper.get_trading_days(self.start_simulate_date, self.end_simulate_date)
            for trading_date in lst_trading_days:
                self.trading_date = trading_date
                self.order_sys_id = 1
                self.trade_sys_id = 1
                self.data_handler.init(trading_date)

                for instrument_id in self.subscribe_futures:
                    tick_series = self.data_handler.load_tick_series(EnumMarket.期货, instrument_id, trading_date)
                    if instrument_id == self.drive_code:
                        self.drive_replay_data.add_data(trading_date, tick_series)
                    else:
                        self.other_replay_data_map[instrument_id].add_data(trading_date, tick_series)

                self.__replay_tick(trading_date)
        elif self.drive_bar_type == EnumBarType.night_am_pm:
            self.on_start()
            lst_trading_days = TradingDayHelper.get_trading_days(self.start_simulate_date, self.end_simulate_date)
            for trading_date in lst_trading_days:
                self.trading_date = trading_date
                self.order_sys_id = 1
                self.trade_sys_id = 1
                self.data_handler.init(trading_date)

                for instrument_id in self.subscribe_futures:
                    bar_series = self.data_handler.load_night_am_pm_bar_series_by_date(
                        EnumMarket.期货, self.drive_code, self.drive_bar_interval, self.drive_bar_type, trading_date, trading_date)
                    if instrument_id == self.drive_code:
                        self.drive_replay_data.add_data(trading_date, bar_series)
                    else:
                        self.other_replay_data_map[instrument_id].add_data(trading_date, bar_series)

                self.__replay_bar(trading_date)

        self.generate_strategy_report()
        self.on_end()
        self.run_seconds = time.time() - t0
        print('策略运行完毕,耗时：{0}秒'.format(self.run_seconds))

    def day_begin(self, trading_date, tick_now):
        """
        交易日开始
        :param trading_date:
        :param tick_now:
        """
        self.tick_now = tick_now
        self.data_handler.trading_day = trading_date
        if self.future_data is None:
            self.future_data = FutureData()
            self.future_data.asset = FutureAsset()
            self.future_data.asset.product_code = self.product_code
            self.future_data.asset.strategy_id = self.strategy_id
            self.future_data.asset.fund_balance = self.fund_balance
            self.future_data.asset.cash = self.fund_balance
            self.future_data.asset.available_cash = self.fund_balance
            self.future_data.asset.pre_fund_balance = self.fund_balance
            self.future_data.asset.trading_date = trading_date
            self.future_data.next_asset = self.future_data.asset
        self.future_cache.day_begin(trading_date, self.future_data)
        self.indicators.clear()
        self.on_init()

    def day_end(self):
        """
        交易日结束
        """
        self.future_data = self.future_cache.day_end()
        self.all_assets.append(self.future_data.asset)
        for order in self.future_data.orders:
            self.all_orders.append(order)
            if order.direction == EnumBuySell.买入:
                direction = '买入'
            else:
                direction = '卖出'
            if order.open_close == EnumOpenClose.开仓:
                open_close = '开仓'
            else:
                open_close = '平仓'
            self.lst_order.append([order.trading_date, order.order_time, order.instrument_id, order.order_price, direction, open_close, order.order_volume,
                                   order.order_status])
        for trade in self.future_data.trades:
            self.all_trades.append(trade)
            if trade.direction == EnumBuySell.买入:
                direction = '买入'
            else:
                direction = '卖出'
            if trade.open_close == EnumOpenClose.开仓:
                open_close = '开仓'
            else:
                open_close = '平仓'
            self.lst_trade.append([trade.trading_date, trade.trade_time, trade.instrument_id, trade.trade_price, direction, open_close, trade.trade_volume])
        for position in self.future_data.positions:
            self.all_positions.append(position)
            if position.direction == EnumPositionDirection.多头:
                direction = '多头'
            else:
                direction = '空头'
            self.lst_position.append([position.trading_date, position.instrument_id, direction, position.volume, position.last_price,
                                      position.position_profit, position.market_value, position.num_holding_day])

        for position_detail in self.future_data.position_details:
            self.all_position_details.append(position_detail)
            if position_detail.direction == EnumPositionDirection.多头:
                direction = '多头'
            else:
                direction = '空头'
            if position_detail.position_type == EnumPositionType.今仓:
                position_type = '今仓'
            else:
                position_type = '昨仓'
            self.lst_position_detail.append([position_detail.trading_date, position_detail.instrument_id, direction, position_detail.volume,
                                             position_detail.open_price, position_detail.open_time, position_type,
                                             position_detail.position_profit, position_detail.float_profit, position_detail.market_value])

        self.lst_asset.append([self.future_data.asset.trading_date, self.future_data.asset.pre_fund_balance, self.future_data.asset.fund_balance,
                               self.future_data.asset.close_profit, self.future_data.asset.position_profit,
                               self.future_data.asset.close_profit + self.future_data.asset.position_profit,
                               self.future_data.asset.available_cash, self.future_data.asset.margin, self.future_data.asset.fee])

    def __replay_bar(self, trading_date):
        """
        播放bar
        :param trading_date:
        """
        self.drive_replay_data.current_trading_date = trading_date
        for replay_data in self.other_replay_data_map.values():
            replay_data.current_trading_date = trading_date

        if trading_date in self.drive_replay_data.data_block_map.keys():
            bar_series = self.drive_replay_data.data_block_map[trading_date].data_list
        else:
            bar_series = None
        try:
            tick_now = bar_series[0].begin_time + datetime.timedelta(seconds=-2)
            trading_date = bar_series[0].trading_date
            self.day_begin(trading_date, tick_now)
            for bar in bar_series:
                bar.instrument_id = self.drive_code
                self.data_handler.on_bar(bar)
                self.last_bar = bar
                self.tick_now = bar.end_time
                self.future_cache.on_bar(bar)
                self.last_bar_map[self.drive_code] = bar
                for indicator in self.indicators:
                    indicator.calculate()

                is_ready = True
                for key in self.other_replay_data_map.keys():
                    res = self.other_replay_data_map[key].replay_bar_to_time(self.tick_now)
                    if res[0]:
                        for other_bar in res[1]:
                            other_bar.instrument_id = key
                            self.data_handler.on_bar(other_bar)
                            self.future_cache.on_bar(other_bar)
                            self.last_bar_map[key] = other_bar
                    else:
                        is_ready = False
                        break
                if is_ready:
                    self.on_bar(bar)

            self.on_exit()
            self.day_end()
        except Exception as e:
            if (bar_series is None) or (len(bar_series) == 0):
                print('{0}@{1},缺失数据'.format(self.drive_code, self.trading_date))
                self.day_begin(self.trading_date, self.trading_date)
                self.on_exit()
                self.day_end()
            else:
                print(str(e))
                raise e

    def __replay_tick(self, tick_series):
        """
        播放tick
        :param tick_series:
        """
        tick_now = tick_series[0].date_time + datetime.timedelta(seconds=-2)
        trading_date = tick_series[0].trading_day
        self.day_begin(trading_date, tick_now)
        for tick in tick_series:
            tick.instrument_id = self.drive_code
            self.data_handler.on_tick(tick)
            self.last_tick = tick
            self.tick_now = tick.date_time
            self.future_cache.on_tick(tick)
            for indicator in self.indicators:
                indicator.calculate()
            self.on_tick(tick)

        self.on_exit()
        self.day_end()

    def send_order(self, instrument_id, buy_sell, open_close, volume, price, order_time=None, remark=''):
        """
        发送委托
        :param remark:
        :param order_time:
        :param instrument_id:
        :param buy_sell:
        :param open_close:
        :param volume:
        :param price:
        """
        order = FutureOrder()
        order.order_sys_id = self.order_sys_id
        order.product_code = self.product_code
        order.strategy_id = self.strategy_id
        order.account_id = instrument_id
        order.instrument_id = instrument_id
        order.direction = buy_sell
        order.open_close = open_close
        order.order_volume = volume
        order.order_price = price
        order.trading_date = self.trading_date
        if order_time is not None:
            order.order_time = order_time
        else:
            if order.instrument_id in self.last_bar_map:
                last_bar = self.last_bar_map[order.instrument_id]
                if last_bar is not None:
                    if last_bar.open == price:
                        order.order_time = self.last_bar.begin_time
                    elif last_bar.close == price:
                        order.order_time = self.last_bar.end_time
                    else:
                        order.order_time = self.tick_now
                else:
                    order.order_time = self.tick_now
            else:
                order.order_time = self.tick_now
        order.order_status = '已成交'
        self.future_cache.on_order(order)
        self.order_sys_id += 1
        self.on_order(order)

        trade = FutureTrade()
        trade.order_sys_id = order.order_sys_id
        trade.trade_sys_id = self.trade_sys_id
        trade.product_code = self.product_code
        trade.strategy_id = self.strategy_id
        trade.account_id = instrument_id
        trade.instrument_id = instrument_id
        trade.direction = buy_sell
        trade.open_close = open_close
        trade.trade_volume = volume
        trade.trade_price = price
        trade.trading_date = self.trading_date
        trade.trade_time = order.order_time
        trade.remark = remark
        self.future_cache.on_trade(trade)
        self.trade_sys_id += 1
        if self.last_bar is not None:
            print('[{0}-{1}][开:{3},收:{4}]:{2}'.format(self.last_bar.begin_time.strftime('%H:%M:%S'),
                                                      self.last_bar.end_time.strftime('%H:%M:%S'), trade.to_string(), self.last_bar.open,
                                                      self.last_bar.close))
        elif self.last_tick is not None:
            print('[{0}]:{1}'.format(self.tick_now.strftime('%H:%M:%S.%f'), trade.to_string()))
        else:
            print('{0}'.format(trade.to_string()))
        self.on_trade(trade)

    @abc.abstractmethod
    def on_start(self):
        """
        回测开始
        """
        pass

    @abc.abstractmethod
    def on_init(self):
        """
        初始化
        """
        pass

    @abc.abstractmethod
    def on_bar(self, bar):
        """
        on_bar触发
        :param bar:
        """
        pass

    @abc.abstractmethod
    def on_tick(self, tick):
        """
        on_tick触发
        :param tick:
        """
        pass

    @abc.abstractmethod
    def on_order(self, order):
        """
        委托回报
        :param order:
        """
        pass

    @abc.abstractmethod
    def on_order_canceled(self, order):
        """
        撤单成功回报
        :param order:
        """
        pass

    @abc.abstractmethod
    def on_cancel_order_failed(self, order_guid, msg):
        """
        撤单失败回报
        :param order_guid:
        :param msg:
        """
        pass

    @abc.abstractmethod
    def on_order_rejected(self, order):
        """
        委托拒绝回报
        :param order:
        """
        pass

    @abc.abstractmethod
    def on_trade(self, trade):
        """
        成交回报
        :param trade:
        """
        pass

    @abc.abstractmethod
    def on_exit(self):
        """
        当日结束
        """
        pass

    @abc.abstractmethod
    def on_end(self):
        """
        回测结束
        """
        pass

    def get_bar_series_by_date_time(self, market, instrument_id, interval, bar_type, begin_time):
        """
        根据时间获取K线
        :param market:
        :param instrument_id:
        :param interval:
        :param bar_type:
        :param begin_time:
        :return:
        """
        return self.data_handler.get_bar_series_by_date_time(market, instrument_id, interval, bar_type, begin_time, self.tick_now)

    def get_bar_series_by_length(self, market: object, instrument_id: object, interval: object, bar_type: object, max_length: object) -> object:
        """
        根据回溯根数获取K线
        :param market:
        :param instrument_id:
        :param interval:
        :param bar_type:
        :param max_length:
        :return:
        """
        return self.data_handler.get_bar_series_by_length(market, instrument_id, interval, bar_type, max_length, self.tick_now)

    def generate_strategy_report(self):
        """
        生成策略报告
        """
        self.df_asset = pd.DataFrame(self.lst_asset, columns=['日期', '昨日结存', '动态权益', '平仓盈亏', '持仓盈亏', '净利润', '可用资金', '持仓保证金', '手续费'])
        self.df_asset['累计盈亏'] = self.df_asset[['净利润']].cumsum(axis=0)
        self.df_asset['累计收益率'] = self.df_asset['累计盈亏'] / self.fund_balance
        self.df_asset['累计收益率'] = self.df_asset['累计收益率'].apply(lambda x: format(x, '.2%'))

        self.df_order = pd.DataFrame(self.lst_order, columns=['交易日', '时间', '合约', '价格', '多空', '开平', '量', '状态'])
        self.df_trade = pd.DataFrame(self.lst_trade, columns=['交易日', '时间', '合约', '价格', '多空', '开平', '量'])
        self.df_position = pd.DataFrame(self.lst_position, columns=['交易日', '合约', '多空', '总持仓', '当前价', '持仓盈亏', '合约价值', '持有天数'])
        self.df_position_detail = pd.DataFrame(self.lst_position_detail, columns=['交易日', '合约', '多空', '总持仓', '开仓价', '开仓时间',
                                                                                  '持仓类型', '持仓盈亏', '浮动盈亏', '持仓市值'])

        self.df_trade['时间'] = self.df_trade['时间'].apply(lambda x: x.strftime('%H:%M:%S'))

        self.trading_day_num = TradingDayHelper.get_trading_day_count(self.start_simulate_date, self.end_simulate_date)
        self.total_trade_lots = self.df_trade['量'].sum() / 2
        self.volume_per_day = self.total_trade_lots / len(self.df_asset)
        self.yield_rate = StrategyToolBox.calc_total_and_annual_yield_rate(self.df_asset, self.fund_balance, 243, self.trading_day_num)
        self.total_yield_rate = self.yield_rate[0]
        self.annual_yield_rate = self.yield_rate[1]
        self.profit_per_hand = StrategyToolBox.calc_profit_per_hand(self.df_asset, self.total_trade_lots)
        self.max_retrace_rate = StrategyToolBox.calc_max_retrace_rate(self.df_asset, self.fund_balance)
        self.max_retrace = self.max_retrace_rate * self.fund_balance
        self.retrace_ratio = StrategyToolBox.calc_retrace_ratio(self.max_retrace_rate, self.annual_yield_rate)
        self.sharpe_ratio = StrategyToolBox.calc_sharpe_ratio(self.df_asset, self.fund_balance)
        self.max_daily_loss = self.df_asset['净利润'].min()
        self.continuous_innovation_high_days = StrategyToolBox.calc_max_continuous_no_innovation_high_days(self.df_asset)
        self.max_loss_days = StrategyToolBox.calc_max_loss_days(self.df_asset)
        self.lst_summary = [
            [self.drive_code, self.start_simulate_date, self.end_simulate_date, self.trading_day_num, self.total_trade_lots, self.volume_per_day,
             self.total_yield_rate, self.annual_yield_rate, self.profit_per_hand, self.max_retrace, self.max_retrace_rate, self.retrace_ratio, self.sharpe_ratio,
             self.max_daily_loss, self.continuous_innovation_high_days, self.max_daily_loss]]
        self.df_summary = pd.DataFrame(self.lst_summary, columns=['品种', '开始日期', '截止日期', '交易天数', '总手数', '日均手数',
                                                                  '总收益', '年化收益', '每手盈利', '最大回撤值', '最大回撤', '收益回撤比', '夏普率',
                                                                  '单日最大亏损', '连续不创新高天数', '最大连续亏损天数'])

        lst_param = []
        for key in self.kwargs:
            if key not in ['factor_map', 'data_handle', 'mongodb_client']:
                lst_param.append([key, str(self.kwargs[key])])
        self.df_param = pd.DataFrame(lst_param, columns=['参数名', '参数值'])

    def save_strategy(self, excel_record_path, fig_record_path):
        """
        保存策略相关回测信息
        """
        try:
            dic_data = {
                'Asset': self.df_asset,
                'Order': self.df_order,
                'Trade': self.df_trade,
                'Position': self.df_position,
                'PositionDetail': self.df_position_detail,
                'Summary': self.df_summary,
                'Param': self.df_param,
            }
            FileToolBox.save_excel(excel_record_path, **dic_data)
            self.df_asset['累计收益率(%)'] = self.df_asset['累计收益率'].str.strip('%').astype(float) / 100
            FileToolBox.save_fig(fig_record_path, self.df_asset, '日期', ['累计收益率(%)'])
        except Exception as e:
            print(str(e))

    def save_mongodb(self, db_name, collection_name, account_id):
        """
        保存记录到数据库
        @param db_name:
        @param collection_name:
        @param account_id:
        """
        try:
            record = {
                'version': 1.0,
                'drive_code': self.drive_code,
                'start_simulate_date': self.start_simulate_date,
                'end_simulate_date': self.end_simulate_date,
                'account_id': account_id,
                '_asset': self.df_asset.to_dict('records'),
                'trade': self.df_trade.to_dict('records'),
                'summary': self.df_summary.to_dict('records')[0],
                'param': self.df_param.set_index(['参数名']).T.to_dict('records')[0],
                'update_time': datetime.datetime.now(),
                'run_seconds': self.run_seconds
            }
            db = self.mongodb_client[db_name]
            collection = db[collection_name]
            collection.insert_one(record)
        except Exception as e:
            print(str(e))
            print('保存数据库出错，自动转存文件格式保存......')
            excel_record_path = os.path.join(self.record_dir, '{0}.xlsx'.format(account_id))
            fig_record_path = os.path.join(self.record_dir, '{0}.png'.format(account_id))
            self.save_strategy(excel_record_path, fig_record_path)



import datetime

from SapphireQuant.Core.Enum.EnumFutureType import EnumFutureType
from SapphireQuant.Core.Enum.EnumMarket import EnumMarket
from SapphireQuant.Core.Instrument import Instrument


class Future(Instrument):
    """
    期货
    """
    def __init__(self):
        super().__init__()
        self._open_date = datetime.datetime.min
        self._future_type = EnumFutureType.normal
        self._real_future = None
        self._expire_date = datetime.datetime.max
        self._long_margin_ratio = 0.0
        self._short_margin_ratio = 0.0

        self._can_sel_today_pos = True
        self._market = EnumMarket.期货

    @property
    def open_date(self):
        """

        :return:
        """
        return self._open_date

    @open_date.setter
    def open_date(self, value):
        self._open_date = value

    @property
    def future_type(self):
        """

        :return:
        """
        return self._future_type

    @future_type.setter
    def future_type(self, value):
        self._future_type = value

    @property
    def real_future(self):
        """

        :return:
        """
        return self._real_future

    @real_future.setter
    def real_future(self, value):
        self._real_future = value

    @property
    def expire_date(self):
        """

        :return:
        """
        return self._expire_date

    @expire_date.setter
    def expire_date(self, value):
        self._expire_date = value

    @property
    def long_margin_ratio(self):
        """

        :return:
        """
        return self._long_margin_ratio

    @long_margin_ratio.setter
    def long_margin_ratio(self, value):
        self._long_margin_ratio = value

    @property
    def short_margin_ratio(self):
        """

        :return:
        """
        return self._short_margin_ratio

    @short_margin_ratio.setter
    def short_margin_ratio(self, value):
        self._short_margin_ratio = value

    @property
    def is_arbitrage(self):
        """

        :return:
        """
        return self.id.startswith("SP")

    def link(self, old_future, new_future):
        """

        :param old_future:
        :param new_future:
        """
        pass

    def clone(self):
        """

        :return:
        """
        future = Future()
        future._exchange_id = self._exchange_id
        future._expire_date = self._expire_date
        future._id = self._id
        future._name = self._name
        future._long_margin_ratio = self._long_margin_ratio
        future._open_date = self._open_date
        future._price_tick = self._price_tick
        future._product_id = self._product_id
        future._short_margin_ratio = self._short_margin_ratio
        future._volume_multiple = self._volume_multiple
        future._last_tick = self._last_tick

        return future

    def to_string(self):
        """

        :return:
        """
        string_builder = ""
        string_builder += ("[" + self._id + "]")
        string_builder += self._name + ","
        string_builder += self._product_id + ","
        string_builder += self._exchange_id + ","
        string_builder += "OpenDate=" + self._open_date.strftime("%Y/%m/%d") + ","
        string_builder += "ExpireDate=" + self._expire_date.strftime("%Y/%m/%d") + ","
        string_builder += "LongMarginRatio=" + str(self._long_margin_ratio) + ","
        string_builder += "ShortMarginRatio=" + str(self._short_margin_ratio) + ","
        string_builder += "VolumeMultiple=" + str(self._volume_multiple) + ","
        string_builder += "PriceTick=" + str(self._price_tick) + ","
        string_builder += "昨收=" + str(self._pre_close) + ","
        return string_builder



import datetime
import os
import re
import xml

import xml.dom.minidom

from SapphireQuant.Config.TradingFrame.TradingFrameHelper import TradingFrameHelper
from SapphireQuant.Core.Futures import FutureExchange
from SapphireQuant.Core.Futures.Future import Future
from SapphireQuant.Core.Futures.FutureProduct import FutureProduct


class FutureManager:
    """
    期货合约管理
    """
    FileName = 'future.xml'
    HisFileName = 'HisFutures.xml'
    DefaultExchangeMap = {
        'CFFEX': ['中国金融交易所', '9:00,10:15,10:30,11:30,13:30,15:00'],
        'SHFE': ['上海期货交易所', '9:00,10:15,10:30,11:30,13:30,15:00'],
        'DCE': ['大连商品交易所', '9:00,10:15,10:30,11:30,13:30,15:00'],
        'CZCE': ['郑州商品交易所', '9:00,10:15,10:30,11:30,13:30,15:00'],
        'SGX': ['新加坡交易所', '9:00,10:15,10:30,11:30,13:30,15:00'],
        'SGE': ['上海黄金交易所', '9:00,10:15,10:30,11:30,13:30,15:00'],
        'INE': ['能源所', '9:00,10:15,10:30,11:30,13:30,15:00'],
    }

    # DefaultProductMap = {
    #     # CFFEX
    #     'IF': '沪深300指数',
    #     'IC': '中证500指数',
    #     'IH': '上证50指数',
    #     'T': '10年期国债',
    #     'TF': '5年期国债',
    #     'TS': '2年期国债',
    #
    #     # SHFE
    #     'ru': '天然橡胶',
    #     'cu': '沪铜',
    #     'zn': '沪锌',
    #     'ni': '沪镍',
    #     'sn': '沪锡',
    #     'rb': '螺纹钢',
    #     'hc': '热轧卷',
    #     'bu': '沥青',
    #     'au': '黄金',
    #     'ag': '白银',
    #     'al': '沪铝',
    #     'pb': '沪铅',
    #     'wr': '线材',
    #     'fu': '燃料油',
    #     'sp': '纸浆',
    #     'ss': '不锈钢',
    #
    #     # DCE
    #     'c': '玉米',
    #     'cs': '玉米淀粉',
    #     'a': '豆一',
    #     'b': '豆二',
    #     'm': '豆粕',
    #     'y': '豆油',
    #     'p': '棕榈油',
    #     'jd': '鸡蛋',
    #     'bb': '胶合板',
    #     'fb': '纤维板',
    #     'l': '聚乙烯',
    #     'v': '聚氯乙烯',
    #     'pp': '聚丙烯',
    #     'j': '焦炭',
    #     'jm': '焦煤',
    #     'i': '铁矿石',
    #     'eg': '乙二醇',
    #     'rr': '粳米',
    #     'eb': '苯乙烯',
    #
    #     #CZCE
    #     'WH': '强麦',
    #     'WH': '强麦',
    # }

    DefaultFutureMap = {
        '9999': '主力合约',
        '9991': '次主力合约',
        '9998': '当月连续',
        '9997': '下月连续',
        '9996': '下季连续',
        '9995': '隔季连续',
        '9990': '指数合约',
    }

    def __init__(self):
        self.__all_exchanges = {}
        self.__all_products = {}
        self.__all_futures = {}
        self.__is_loaded = False
        self.__path = ""
        self.__history_path = ""

    @property
    def all_exchanges(self):
        """
        所有交易市场列表
        :return:
        """
        return self.__all_exchanges

    @property
    def all_products(self):
        """
        所有期货产品列表
        :return:
        """
        return self.__all_products

    @property
    def all_instruments(self):
        """
        所有期货合约列表
        :return:
        """
        return self.__all_futures

    def __getitem__(self, item):
        if item.id in self.__all_futures.keys():
            return self.__all_futures[item.id]
        return None

    def load(self, config_directory, load_history=True):
        """
        加载配置文件
        :param load_history:
        :param config_directory:
        """
        if not self.__is_loaded:
            self.load_future(config_directory)
            if load_history:
                self.load_his_future(config_directory)

    def load_future(self, config_directory):
        """
        加载期货Xml
        :param config_directory:
        """
        try:
            self.__path = os.path.join(config_directory, self.FileName)
            self.__history_path = os.path.join(config_directory, self.HisFileName)
            if os.path.exists(self.__path):
                pass
            else:
                print("未找到配置文件:" + self.__path)
            _root = xml.dom.minidom.parse(self.__path).documentElement
            exchange_nodes = _root.getElementsByTagName('Exchange')
            for exchange_node in exchange_nodes:
                self.__read_exchange(exchange_node)

            self.__is_loaded = True
        except Exception as e:
            print(str(e))

    def load_his_future(self, config_directory):
        """
        加载历史期货Xml
        :param config_directory:
        """
        try:
            self.__path = os.path.join(config_directory, self.FileName)
            self.__history_path = os.path.join(config_directory, self.HisFileName)
            if os.path.exists(self.__history_path):
                pass
            else:
                print("未找到配置文件:" + self.__history_path)
            _root = xml.dom.minidom.parse(self.__history_path).documentElement
            exchange_nodes = _root.getElementsByTagName('Exchange')
            for exchange_node in exchange_nodes:
                self.__read_exchange(exchange_node)

            self.__is_loaded = True
        except Exception as e:
            print(str(e))

    def __read_exchange(self, exchange_node):
        exchange = FutureExchange()
        exchange.exchange_id = exchange_node.getAttribute('id')
        exchange.exchange_name = exchange_node.getAttribute('name')

        trading_time_nodes = exchange_node.getElementsByTagName('TradingTime')
        trading_time = self.get_node_value(trading_time_nodes[0])
        exchange.all_slice = TradingFrameHelper.parse_time_slice(trading_time)
        # trading_time_arr = trading_time.split(',')
        # count = len(trading_time_arr) / 2
        # for i in range(int(count)):
        #     time_slice = TimeSlice()
        #     time_slice.begin_time = TradingFrameHelper.str_pares_timedelta(trading_time_arr[i * 2])
        #     time_slice.end_time = TradingFrameHelper.str_pares_timedelta(trading_time_arr[i * 2 + 1])
        #
        #     exchange.all_slice.append(time_slice)

        open_time_nodes = exchange_node.getElementsByTagName('OpenTime')
        open_timedelta = TradingFrameHelper.str_pares_timedelta(self.get_node_value(open_time_nodes[0]))
        exchange.open_time = datetime.date.today() + open_timedelta

        close_time_nodes = exchange_node.getElementsByTagName('CloseTime')
        close_timedelta = TradingFrameHelper.str_pares_timedelta(self.get_node_value(close_time_nodes[0]))
        exchange.close_time = datetime.date.today() + close_timedelta

        self.__add_exchange(exchange)

        product_nodes = exchange_node.getElementsByTagName('Product')
        for product_node in product_nodes:
            self.__read_product(product_node, exchange.exchange_id)

    def __read_product(self, product_node, exchange_id):
        product = FutureProduct()
        product.product_id = product_node.getAttribute('id')
        product.product_name = product_node.getAttribute('name')
        product.exchange_id = exchange_id

        if product.product_id != "":
            self.__add_product(product)

        future_nodes = product_node.getElementsByTagName('Future')
        for future_node in future_nodes:
            future = self.__read_future(future_node)
            future_id = future.id
            if (future_id.find("9995") > 0) | (future_id.find("9996") > 0) | (future_id.find("9997") > 0) | (future_id.find("9998") > 0) | (
                    future_id.find("9999") > 0):
                future.real_future = future

            if future.id != "":
                future.exchange_id = product.exchange_id
                future.product_id = product.product_id

                self.__add_future(future)

    def __read_future(self, future_node):
        future = Future()
        future.id = future_node.getAttribute('id')
        future.name = future_node.getAttribute('name')

        product_id_nodes = future_node.getElementsByTagName('ProductID')
        future.product_id = self.get_node_value(product_id_nodes[0])

        exchange_id_nodes = future_node.getElementsByTagName('ExchangeID')
        future.exchange_id = self.get_node_value(exchange_id_nodes[0])

        open_date_nodes = future_node.getElementsByTagName('OpenDate')
        future.open_date = datetime.datetime.strptime(self.get_node_value(open_date_nodes[0]), '%Y/%m/%d %H:%M:%S')

        expire_date_nodes = future_node.getElementsByTagName('ExpireDate')
        future.expire_date = datetime.datetime.strptime(self.get_node_value(expire_date_nodes[0]), '%Y/%m/%d %H:%M:%S')

        long_margin_ratio_nodes = future_node.getElementsByTagName('LongMarginRatio')
        future.long_margin_ratio = float(self.get_node_value(long_margin_ratio_nodes[0]))

        short_margin_ratio_nodes = future_node.getElementsByTagName('ShortMarginRatio')
        future.short_margin_ratio = float(self.get_node_value(short_margin_ratio_nodes[0]))

        volume_multiple_nodes = future_node.getElementsByTagName('VolumeMultiple')
        future.volume_multiple = int(self.get_node_value(volume_multiple_nodes[0]))

        price_tick_nodes = future_node.getElementsByTagName('PriceTick')
        future.price_tick = float(self.get_node_value(price_tick_nodes[0]))

        return future

    @staticmethod
    def get_node_value(node):
        """
        获取节点的值
        :param node:
        :return:
        """
        return node.firstChild.data

    def __add_exchange(self, exchange):
        if exchange.exchange_id in self.__all_exchanges.keys():
            return

        self.__all_exchanges[exchange.exchange_id] = exchange

        # print(exchange.to_string())

    def __add_product(self, product):
        if product.product_id in self.__all_products.keys():
            return

        self.__all_products[product.product_id] = product

        if product.exchange_id in self.__all_exchanges.keys():
            exchange = self.__all_exchanges[product.exchange_id]
            if product.product_id not in exchange.product_map.keys():
                exchange.products.append(product)
                exchange.product_map[product.product_id] = product

        # print(product.to_string())

    def __add_future(self, future):
        if future is None:
            return

        if future.id in self.__all_futures.keys():
            return

        self.__all_futures[future.id] = future

        if future.product_id in self.__all_products.keys():
            product = self.__all_products[future.product_id]
            if future.id not in product.future_map.keys():
                product.futures.append(future)
                product.future_map[future.id] = future

        # print(future.to_string())

    @staticmethod
    def get_default_exchanges():
        """
        获取默认的Exchange
        """
        dic_exchanges = {}
        for key in FutureManager.DefaultExchangeMap.keys():
            exchange = FutureExchange()
            exchange.exchange_id = key
            exchange.exchange_name = FutureManager.DefaultExchangeMap[key][0]
            trading_time = FutureManager.DefaultExchangeMap[key][1]
            exchange.all_slice = TradingFrameHelper.parse_time_slice(trading_time)
            exchange.open_time = exchange.all_slice[0].begin_time
            exchange.close_time = exchange.all_slice[-1].end_time
            dic_exchanges[key] = exchange
        return dic_exchanges

    @staticmethod
    def get_default_futures():
        """
        获取默认的Future
        """
        dic_futures = {}
        for key in FutureManager.DefaultFutureMap.keys():
            future = Future()
            future.id = key
            future.name = FutureManager.DefaultFutureMap[key]
            future.product_id = None
            future.exchange_id = None
            future.open_date = datetime.datetime.min
            future.expire_date = datetime.datetime.max
            future.long_margin_ratio = 0
            future.short_margin_ratio = 0
            future.volume_multiple = 0
            future.price_tick = 0
            dic_futures[key] = future

    def update(self, lst_futures):
        """
        更新期货列表
        :param lst_futures:
        """
        history_exchanges = FutureManager.get_default_exchanges()
        today_exchanges = FutureManager.get_default_exchanges()
        lst_today_future_id = list(map(lambda x: x.id, lst_futures))
        # 原有期货品种分成历史和当日
        for future_id in self.__all_futures.keys():
            default_id = re.sub(r"[a-zA-Z]", "", future_id, 0)
            if default_id in FutureManager.DefaultFutureMap.keys():
                continue
            future = self.__all_futures[future_id]
            target_exchange = None
            if future_id in lst_today_future_id:
                if future.exchange_id in today_exchanges.keys():
                    target_exchange = today_exchanges[future.exchange_id]
                else:
                    print('非法的交易市场ID:{0}'.format(future.exchange_id))
            else:
                if future.exchange_id in history_exchanges.keys():
                    target_exchange = history_exchanges[future.exchange_id]
                else:
                    print('非法的交易市场ID:{0}'.format(future.exchange_id))

            if target_exchange is not None:
                if future.product_id in target_exchange.product_map.keys():
                    product = target_exchange.product_map[future.product_id]
                else:
                    product = FutureProduct()
                    product.product_id = future.product_id
                    product.product_name = re.sub(r'[0-9]*$', '', future.name, 0)
                    target_exchange.product_map[future.product_id] = product
                    target_exchange.products.append(product)
                if future_id in product.future_map.keys():
                    product.future_map[future_id] = future
                else:
                    product.future_map[future_id] = future
                    product.futures.append(future)

        for future in lst_futures:
            if future.exchange_id in today_exchanges.keys():
                exchange = today_exchanges[future.exchange_id]
            else:
                exhcange = FutureExchange()
                exchange.exchange_id = future.exchange_id
                exchange.exchange_name = future.exchange_id
                today_exchanges[exchange.exchange_id] = exchange
            if future.product_id in exchange.product_map.keys():
                product = exchange.product_map[future.product_id]
            else:
                product = FutureProduct()
                product.product_id = future.product_id
                product.exchange_id = future.exchange_id
                product.product_name = future.product_id
                exchange.product_map[product.product_id] = product
                exchange.products.append(product)

            if future.id not in product.future_map.keys():
                product.future_map[future.id] = future
                product.futures.append(future)

        for exchange in today_exchanges.values():
            for product in exchange.products:
                for custom_id in FutureManager.DefaultFutureMap.keys():
                    custom_future = Future()
                    custom_future.exchange_id = exchange.exchange_id
                    custom_future.product_id = product.product_id
                    custom_future.id = '{0}{1}'.format(product.product_id, custom_id)
                    custom_future.name = '{0}{1}'.format(product.product_name, FutureManager.DefaultFutureMap[custom_id])
                    if len(product.futures) > 0:
                        product.futures.sort(key=lambda x: x.expire_date, reverse=True)
                        future = product.futures[0]
                        custom_future.expire_date = future.expire_date
                        custom_future.open_date = future.open_date
                        custom_future.long_margin_ratio = future.long_margin_ratio
                        custom_future.short_margin_ratio = future.short_margin_ratio
                        custom_future.volume_multiple = future.volume_multiple
                        custom_future.price_tick = future.price_tick
                        product.futures.append(custom_future)
                        product.future_map[custom_future.id] = custom_future

        FutureManager.save_xml(today_exchanges, self.__path)
        FutureManager.save_xml(history_exchanges, self.__history_path)

    @staticmethod
    def save_xml(exchanges, file_path):
        """
        保存文件
        :param exchanges:
        :param file_path:
        """
        dom = xml.dom.minidom.getDOMImplementation().createDocument(None, 'Ats', None)
        root = dom.documentElement
        lst_exchange = list(exchanges.values())
        lst_exchange.sort(key=lambda x: x.exchange_id)
        for exchange in lst_exchange:
            exchange_element = dom.createElement('Exchange')
            exchange_element.setAttribute('id', str(exchange.exchange_id))
            exchange_element.setAttribute('name', str(exchange.exchange_name))
            exchange_element.setAttribute('count', str(len(exchange.products)))

            trading_time = ''
            for time_slice in exchange.all_slice:
                trading_time = '{0},{1},{2}'.format(trading_time, str(time_slice.begin_time), str(time_slice.end_time))
            trading_time = trading_time[1:]
            open_time = str(exchange.all_slice[0].begin_time)
            close_time = str(exchange.all_slice[-1].end_time)
            trading_time_element = dom.createElement('TradingTime')
            trading_time_element.appendChild(dom.createTextNode(trading_time))
            open_time_element = dom.createElement('OpenTime')
            open_time_element.appendChild(dom.createTextNode(open_time))
            close_time_element = dom.createElement('CloseTime')
            close_time_element.appendChild(dom.createTextNode(close_time))
            exchange_element.appendChild(trading_time_element)
            exchange_element.appendChild(open_time_element)
            exchange_element.appendChild(close_time_element)

            lst_product = exchange.products
            lst_product.sort(key=lambda x: x.product_id)
            for product in lst_product:
                product_element = dom.createElement('Product')
                product_element.setAttribute('id', str(product.product_id))
                product_element.setAttribute('name', str(product.product_name))
                exchange_element.appendChild(product_element)

                lst_future = product.futures
                lst_future.sort(key=lambda x: x.id, reverse=True)
                for future in lst_future:
                    future_element = dom.createElement('Future')
                    future_element.setAttribute('id', future.id)
                    future_element.setAttribute('name', future.name)

                    future_exchange_element = dom.createElement('ExchangeID')
                    future_exchange_element.appendChild(dom.createTextNode(future.exchange_id))

                    future_expire_date_element = dom.createElement('ExpireDate')
                    future_expire_date_element.appendChild(dom.createTextNode(future.expire_date.strftime('%Y/%m/%d %H:%M:%S')))

                    future_long_margin_ratio_element = dom.createElement('LongMarginRatio')
                    future_long_margin_ratio_element.appendChild(dom.createTextNode(str(future.long_margin_ratio)))

                    future_open_date_element = dom.createElement('OpenDate')
                    future_open_date_element.appendChild(dom.createTextNode(future.open_date.strftime('%Y/%m/%d %H:%M:%S')))

                    future_product_id_element = dom.createElement('ProductID')
                    future_product_id_element.appendChild(dom.createTextNode(future.product_id))

                    future_short_margin_ratio_element = dom.createElement('ShortMarginRatio')
                    future_short_margin_ratio_element.appendChild(dom.createTextNode(str(future.short_margin_ratio)))

                    future_volume_multiple_element = dom.createElement('VolumeMultiple')
                    future_volume_multiple_element.appendChild(dom.createTextNode(str(future.volume_multiple)))

                    future_price_tick_element = dom.createElement('PriceTick')
                    future_price_tick_element.appendChild(dom.createTextNode(str(future.price_tick)))

                    future_element.appendChild(future_exchange_element)
                    future_element.appendChild(future_expire_date_element)
                    future_element.appendChild(future_long_margin_ratio_element)
                    future_element.appendChild(future_open_date_element)
                    future_element.appendChild(future_product_id_element)
                    future_element.appendChild(future_short_margin_ratio_element)
                    future_element.appendChild(future_volume_multiple_element)
                    future_element.appendChild(future_price_tick_element)
                    product_element.appendChild(future_element)

            root.appendChild(exchange_element)

        with open(file_path, 'w', encoding='utf-8') as f:
            dom.writexml(f, addindent='\t', newl='\n', encoding='utf-8')

# future_manager = FutureManager()
# config_dir = os.path.join(sys.modules["ROOT_DIR"], 'Profiles')
# future_manager.load(config_dir, load_history=False)
# lst_futures = future_manager.all_products['IF'].futures
# future_manager.update(lst_futures)
# for data in future_manager.all_instruments.values():
#     print(data.to_string())
# print(re.sub(r'[0-9]*$', '', '沪深300指数2008', 0))



import datetime

from SapphireQuant.Config.TradingDay import TradingDayHelper
from SapphireQuant.Core import QiLogger
from SapphireQuant.Core.Enum import EnumMarket, EnumBarType
from SapphireQuant.DataProcessing.BaseBarHelper import BaseBarHelper
from SapphireQuant.DataProcessing.QiDataController import QiDataController
from SapphireQuant.DataProcessing.QiDataDirectory import QiDataDirectory


class DataCheck:
    """
    数据检查
    """

    @staticmethod
    def check_bar_series(data_handler, instrument_ids, exchange_id, begin_date, end_date, log_dir=None):
        """
        检查K线
        :param data_handler:
        :param end_date:
        :param begin_date:
        :param log_dir:
        :param instrument_ids:
        :param exchange_id:
        """
        log = None
        if log_dir is not None:
            log = QiLogger.create_log(log_dir, '{0}.log'.format(exchange_id))
        if log is not None:
            log.info('开始检查:{0}个品种,[{1}-{2}]:{3}'.format(len(instrument_ids), begin_date.strftime('%Y%m%d'), end_date.strftime('%Y%m%d'), instrument_ids))
        for instrument_id in instrument_ids:
            listing_date = data_handler.instrument_manager.get_listing_date(EnumMarket.期货, instrument_id)
            if listing_date > begin_date:
                begin_date = listing_date
                if log is not None:
                    log.info('[{0}][{1}]:begin_date切换：{2}'.format(exchange_id, instrument_id, begin_date))
            if log is not None:
                log.info('开始检查:{0},[{1}-{2}]'.format(instrument_id, begin_date.strftime('%Y%m%d'), end_date.strftime('%Y%m%d')))
            lst_trading_days = TradingDayHelper.get_trading_days(begin_date, end_date)
            for trading_date in lst_trading_days:
                print('开始检查:{0},[{1}]'.format(instrument_id, trading_date.strftime('%Y%m%d')))
                df_bar_series = data_handler.load_min_bar_data_frame(EnumMarket.期货, instrument_id, trading_date)
                df_bar_series_standard = BaseBarHelper.load_data_frame_in_day_date_time_slice_by_date(data_handler.instrument_manager,
                                                                                                      instrument_id, trading_date, trading_date, 1, EnumBarType.minute)
                if (df_bar_series is None) or len(df_bar_series) == 0:
                    if log is not None:
                        log.error('[{0}][{1}][{2}]:缺少数据'.format(exchange_id, instrument_id, trading_date))
                else:
                    last_bar_end_time = df_bar_series.loc[len(df_bar_series) - 1, 'end_time']
                    last_bar_end_time_delta = datetime.timedelta(hours=last_bar_end_time.hour, minutes=last_bar_end_time.minute,
                                                                 seconds=last_bar_end_time.second, microseconds=last_bar_end_time.microsecond)
                    if datetime.timedelta(hours=14, minutes=59, seconds=1) < last_bar_end_time_delta < datetime.timedelta(hours=15, minutes=0, seconds=59):
                        df_bar_series.loc[len(df_bar_series) - 1, 'end_time'] = datetime.datetime(last_bar_end_time.year, last_bar_end_time.month, last_bar_end_time.day, 15, 0, 0)
                    if datetime.timedelta(hours=15, minutes=14, seconds=1) < last_bar_end_time_delta < datetime.timedelta(hours=15, minutes=15, seconds=59):
                        df_bar_series.loc[len(df_bar_series) - 1, 'end_time'] = datetime.datetime(last_bar_end_time.year, last_bar_end_time.month, last_bar_end_time.day, 15, 15, 0)

                    df_combine = df_bar_series.merge(df_bar_series_standard, on=['begin_time', 'end_time'], how='outer')
                    nan_rows = df_combine[df_combine.isnull().T.any().T]
                    if len(nan_rows) > 0:
                        if log is not None:
                            log.error('[{0}][{1}][{2}]:缺少数据'.format(exchange_id, instrument_id, trading_date))
                            log.info(nan_rows)
            print('[{0}][{1}]:检查完毕，共计检查{2}个交易日'.format(exchange_id, instrument_id, len(lst_trading_days)))

    @staticmethod
    def check_bar_series_by_exchange(data_handler, exchange_id, begin_date, end_date, log_dir=None):
        """
        按市场检查K线，检查市场内所有品种
        :param data_handler:
        :param exchange_id:
        :param begin_date:
        :param end_date:
        :param log_dir:
        """
        product_ids = data_handler.instrument_manager.get_product_ids(EnumMarket.期货, exchange_id)
        lst_instrument_id = []
        for product_id in product_ids:
            lst_instrument_id.append('{0}9999'.format(product_id))
        DataCheck.check_bar_series(data_handler, lst_instrument_id, exchange_id, begin_date, end_date, log_dir)


if __name__ == '__main__':
    server_ip = '192.168.1.200'

    qi_data_directory = QiDataDirectory()
    qi_data_directory.future_tick = "\\\\{0}\\MqData\\futuretick\\Future".format(server_ip)
    qi_data_directory.future_tick_cache = "\\\\{0}\\MqData\\futuretick\\Future".format(server_ip)
    qi_data_directory.future_min = "\\\\{0}\\MqData\\futuremin".format(server_ip)
    qi_data_directory.future_day = "\\\\{0}\\MqData\\futureday".format(server_ip)
    qi_data_directory.trading_day = datetime.datetime(3020, 1, 1)
    data_handler = QiDataController(qi_data_directory)

    log_dir = 'D:\\DataCheck\\{0}'.format(datetime.datetime.today().strftime('%Y%m%d%H%M%S'))
    begin_date = datetime.datetime(2018, 1, 1)
    end_date = datetime.datetime(2020, 5, 10)


    # begin_date = datetime.datetime(2015, 12, 17)
    # end_date = datetime.datetime(2015, 12, 17)
    # DataCheck.check_bar_series(data_handler, ['IH9999'], 'CFFEX', begin_date, end_date, log_dir)

    DataCheck.check_bar_series_by_exchange(data_handler, 'CFFEX', begin_date, end_date, log_dir)
    DataCheck.check_bar_series_by_exchange(data_handler, 'SHFE', begin_date, end_date, log_dir)
    DataCheck.check_bar_series_by_exchange(data_handler, 'DCE', begin_date, end_date, log_dir)
    DataCheck.check_bar_series_by_exchange(data_handler, 'CZCE', begin_date, end_date, log_dir)
    DataCheck.check_bar_series_by_exchange(data_handler, 'INE', begin_date, end_date, log_dir)

    # p = Pool(10)
    # p.apply_async(data_checker.check_bar_series, args=[['MA9999'], 'CZCE', datetime.datetime(2019, 1, 1), datetime.datetime.today()])
    # p.apply_async(check, args=[instrument_ids_CFFEX, 'CFFEX'])
    # p.apply_async(check, args=[instrument_ids_SHFE, 'SHFE'])
    # p.apply_async(check, args=[instrument_ids_DCE, 'DCE'])
    # p.apply_async(check, args=[instrument_ids_CZCE, 'CZCE'])
    # p.apply_async(check, args=[instrument_ids_INE, 'INE'])
    # p.close()
    # p.join()



from SapphireQuant.Core.Enum import EnumSimulateMatchMode, EnumOrderStatus, EnumOpenClose, EnumBuySell, EnumPositionDirection, EnumTradeEventType
from SapphireQuant.Simulate.Match.LastPriceMatchMachine import LastPriceMatchMachine
from SapphireQuant.Simulate.Match.OrderPriceMatchMachine import OrderPriceMatchMachine


class MatchCenter:
    """
    撮合机
    """

    def __init__(self, future_cache, match_mode, long_margin_rate, short_margin_rate):
        self.on_trade_event = []
        self.on_order_event = []
        self.on_order_reject_event = []
        self.on_order_canceled_event = []
        self.on_order_cancel_failed_event = []

        self._future_cache = future_cache
        self._long_margin_rate = long_margin_rate
        self._short_margin_rate = short_margin_rate
        self._machine = None
        self._trading_date = None
        self._current_time = None
        self._order_serial = 0
        self._order_ref = 0
        if match_mode == EnumSimulateMatchMode.last_price:
            self._machine = LastPriceMatchMachine()
        else:
            self._machine = OrderPriceMatchMachine()

    def get_can_cancel_orders(self):
        """
        获取可撤单列表
        """
        self._machine.get_can_cancel_orders()

    def on_tick(self, tick):
        """

        :param tick:
        """
        self._trading_date = tick.trading_date
        self._current_time = tick.date_time

    def on_bar(self, bar):
        """

        :param bar:
        """
        self._trading_date = bar.trading_date
        self._current_time = bar.end_time

    def match_by_bar(self, bar):
        """

        :param bar:
        """
        self._trading_date = bar.trading_date
        self._current_time = bar.end_time
        lst_res = self._machine.match_by_bar(bar)
        for data in lst_res:
            self._on_order(data[0])
            self._on_trade(data[1])

    def match_by_tick(self, tick):
        """

        :param tick:
        """
        self._trading_date = tick.trading_date
        self._current_time = tick.date_time
        lst_res = self._machine.match_by_tick(tick)
        for data in lst_res:
            self._on_order(data[0])
            self._on_trade(data[1])

    def enter(self, order):
        """
        报单
        :param order:
        """
        self._order_serial += 1
        order.order_sys_id = '{0}'.format(self._order_serial)
        order.trading_date = self._trading_date
        if order.order_time is None:
            order.order_time = self._current_time
        if order.local_order_time is None:
            order.local_order_time = self._current_time
        order.order_status = EnumOrderStatus.未报
        self._on_order(order)
        if self._pre_check(order):
            self._machine.enter(order)
            order.order_status = EnumOrderStatus.已报
            self._on_order(order)

    def cancel_order(self, order):
        """
        撤单
        :param order:
        :return:
        """
        result = self._machine.cancel(order)
        success = result[0]
        order = result[1]
        msg = result[2]
        if success:
            order.volume_canceled = order.order_volume - order.volume_traded
            order.order_status = EnumOrderStatus.已撤
            order.trading_date = self._trading_date
            if order.cancel_time is None:
                order.cancel_time = self._current_time
            self._on_order_canceled(order)
            return True

        order.order_status = EnumOrderStatus.全部成交
        self._on_order_cancel_failed(order.OrderSysId, msg)
        return False

    def _pre_check(self, order):
        if order.open_close == EnumOpenClose.开仓:
            asset = self._future_cache.get_future_asset()
            if order.direction == EnumBuySell.买入:
                margin_rate = self._long_margin_rate
            else:
                margin_rate = self._short_margin_rate

            need_cash = order.order_price * order.order_volume * self._future_cache.instrument_manager[order.instrument_id].volume_multiple * margin_rate
            if asset.available_cash < need_cash:
                order.remark = '综合交易平台：资金不足'
                order.order_status = EnumOrderStatus.废单
                self._on_order_rejected(order)
                return False
        else:
            # 平仓要看反向持仓
            if order.direction == EnumBuySell.买入:
                position_direction = EnumPositionDirection.空头
            else:
                position_direction = EnumPositionDirection.多头

            pos = self._future_cache.get_position(order.product_code, order.strategy_id, order.instrument_id, position_direction)
            if pos is None:
                open_close = order.open_close.chiese_name()
                order.remark = '综合交易平台：{0}仓位不足'.format(open_close)
                self._on_order_rejected(order)
                return False
        return True

    def _on_trade(self, trade):
        for event in self.on_trade_event:
            print(trade.to_string())
            event(trade)

    def _on_order(self, order):
        for event in self.on_order_event:
            # if order.order_status == EnumOrderStatus.全部成交:
            #     print(order.to_string())
            event(order)

    def _on_order_rejected(self, order):
        for event in self.on_order_reject_event:
            event(order)

    def _on_order_canceled(self, order):
        for event in self.on_order_canceled_event:
            event(order)

    def _on_order_cancel_failed(self, order_sys_id, msg):
        for event in self.on_order_cancel_failed_event:
            event(order_sys_id, msg)

    def register_event(self, function, event_type):
        """
        注册事件
        :param function:
        :param event_type:
        """
        if event_type == EnumTradeEventType.on_order:
            self.on_order_event.append(function)
        elif event_type == EnumTradeEventType.on_trade:
            self.on_trade_event.append(function)
        elif event_type == EnumTradeEventType.on_order_reject:
            self.on_order_reject_event.append(function)
        elif event_type == EnumTradeEventType.on_order_cancel_failed:
            self.on_order_cancel_failed_event.append(function)
        elif event_type == EnumTradeEventType.on_order_canceled:
            self.on_order_canceled_event.append(function)

    def send_future_order(self, logic_account_name, order):
        """
        发送委托
        :param logic_account_name:
        :param order:
        :return:
        """
        self._order_ref += 1
        order.local_id = str(self._order_ref)
        self.enter(order)
        return order

    def get_future_asset(self):
        """

        :return:
        """
        return self._future_cache.get_future_asset()

    def get_future_orders(self):
        """

        :return:
        """
        return self._future_cache.get_future_orders()

    def get_future_trades(self):
        """

        :return:
        """
        return self._future_cache.get_future_trades()

    def get_future_positions(self):
        """

        :return:
        """
        return self._future_cache.get_future_positions()

    def get_future_position_details(self):
        """

        :return:
        """
        return self._future_cache.get_future_position_details()

    def get_all_future_assets(self):
        """
        获取所有的期货的账户明细
        :return:
        """
        return self._future_cache.df_asset

    def get_all_future_orders(self):
        """
        获取所有的期货委托
        :return:
        """
        return self._future_cache.df_order

    def get_all_future_trades(self):
        """
        获取所有的期货成交
        :return:
        """
        return self._future_cache.df_trade

    def get_all_future_positions(self):
        """
        获取所有的期货持仓
        :return:
        """
        return self._future_cache.df_position

    def get_all_future_position_details(self):
        """
        获取所有的持仓明细
        :return:
        """
        return self._future_cache.df_position_detail



from SapphireQuant.Core.QiList import QiList


class BarSeries(QiList):
    """
    BarSeries
    """

    def __init__(self):
        self._instrumentId = ''
        super().__init__(self)

    @property
    def instrument_id(self):
        """
        合约编号
        :return:
        """
        return self._instrumentId

    @instrument_id.setter
    def instrument_id(self, value):
        self._instrumentId = value

    def get_item_by_date_time(self, date_time):
        """
        根据时间获取元素
        :param date_time:
        :return:
        """
        count = self.length
        for i in range(0, count):
            bar = self[i]
            if bar.begin_time <= date_time < bar.end_time:
                return bar

        return None

    def get_item_by_index_bar_data(self, index, bar_data):
        """
        根据时间获取元素
        :param index:
        :param bar_data:
        :return:
        """
        if self.length <= index or index < 0:
            return 0.0
        return self[index][bar_data]

    def get_item_by_date_time_bar_data(self, date_time, bar_data):
        """
        根据时间获取元素
        :param date_time:
        :param bar_data:
        :return:
        """
        bar = self.get_item_by_date_time(date_time)
        if bar is not None:
            return bar[bar_data]
        return 0.0

    @property
    def sum_volume(self):
        """
        volume求和
        :return:
        """
        num = 0.0
        for i in range(0, self.length):
            num += self[i].volume
        return num

    @property
    def sum_turnover(self):
        """
        volume求和
        :return:
        """
        num = 0.0
        for i in range(0, self.length):
            num += self[i].turnover
        return num

    @property
    def high(self):
        """
        最高价序列
        :return:
        """
        result = None
        try:
            count = self.length
            data_series = []
            if count > 0:
                for i in range(0, count):
                    data_series.append(self[i].high)
                result = data_series
        except Exception as e:
            result = None
            print(str(e))
        return result

    @property
    def open(self):
        """
        最高价序列
        :return:
        """
        result = None
        try:
            count = self.length
            data_series = []
            if count > 0:
                for i in range(0, count):
                    data_series.append(self[i].open)
                result = data_series
        except Exception as e:
            result = None
            print(str(e))
        return result

    @property
    def low(self):
        """
        最高价序列
        :return:
        """
        result = None
        try:
            count = self.length
            data_series = []
            if count > 0:
                for i in range(0, count):
                    data_series.append(self[i].low)
                result = data_series
        except Exception as e:
            result = None
            print(str(e))
        return result

    @property
    def close(self):
        """
        最高价序列
        :return:
        """
        result = None
        try:
            count = self.length
            data_series = []
            if count > 0:
                for i in range(0, count):
                    data_series.append(self[i].close)
                result = data_series
        except Exception as e:
            result = None
            print(str(e))
        return result

    @property
    def volume(self):
        """
        最高价序列
        :return:
        """
        result = None
        try:
            count = self.length
            data_series = []
            if count > 0:
                for i in range(0, count):
                    data_series.append(self[i].volume)
                result = data_series
        except Exception as e:
            result = None
            print(str(e))
        return result

    @property
    def turnover_series(self):
        """
        最高价序列
        :return:
        """
        result = None
        try:
            count = self.length
            data_series = []
            if count > 0:
                for i in range(0, count):
                    data_series.append(self[i].turnover)
                result = data_series
        except Exception as e:
            result = None
            print(str(e))
        return result

    def contains(self, date_time):
        """
        是否包含指定时间的Bar
        :param date_time:
        :return:
        """
        return self.first is not None and self.last is not None and self.first.begin_time <= date_time <= self.last.end_time

    def clone(self):
        """
        克隆
        :return:
        """
        bar_series = BarSeries()
        for i in range(0, self.length):
            bar_series.append(self[i].clone())
        return bar_series

    def get_date_time(self, index):
        """
        获取指定index对应的Bar的begin_time
        :param index:
        :return:
        """
        if self.length > index:
            return self[index].begin_time
        return None

    def get_index(self, date_time):
        """
        获取指定date_time所在的index
        :param date_time:
        :return:
        """
        count = self.length
        result = -1
        if self.contains(date_time):
            for i in range(0, count):
                if self[i].begin_time <= date_time < self[i].end_time:
                    result = i
        return result

    def get_index_by_option(self):
        """
        pass
        """
        pass



bar_series = BarSeries()
bar_series.append(['1'])


import os

import pandas as pd

from SapphireQuant.Lib.FileToolBox import FileToolBox


class MongoHelper:
    """
    MongoDbHelper
    """

    @staticmethod
    def export_factor_record(db_client, target_dir, factor_id):
        """
        根据因子ID、合约编号导出指定的数据集合，从min_factor库中
        :param target_dir:导出目录
        :param factor_id:因子ID
        :param db_client: Mongo客户端
        """
        factor_record_db = db_client.min_factor
        factor_record_collection = factor_record_db[str(factor_id)]
        factor_record_list = factor_record_collection.find()
        for factor_record in factor_record_list:
            df_asset = pd.DataFrame(factor_record['asset'])
            df_trade = pd.DataFrame(factor_record['trade'])
            df_summary = pd.DataFrame([factor_record['summary']])
            df_param = pd.DataFrame([factor_record['param']]).T
            account_id = factor_record['account_id']

            try:
                excel_record_path = '{0}/{1}/{2}.xlsx'.format(target_dir, factor_id, account_id)
                if not os.path.exists(target_dir):
                    os.makedirs(target_dir)

                dic_data = {
                    'Asset': df_asset,
                    'Trade': df_trade,
                    'Summary': df_summary,
                    'Param': df_param,
                }
                FileToolBox.save_excel(excel_record_path, **dic_data)
                fig_record_path = os.path.join(target_dir, factor_id, '{0}.png'.format(account_id))
                df_asset['累计收益率(%)'] = df_asset['累计收益率'].str.strip('%').astype(float) / 100
                FileToolBox.save_fig(fig_record_path, df_asset, '日期', ['累计收益率(%)'])
            except Exception as e:
                print(str(e))

    @staticmethod
    def export_factor_record_by_instrument_id(db_client, target_dir, factor_id, instrument_id):
        """
        根据因子ID、合约编号导出指定的数据集合，从min_factor库中
        :param target_dir:导出目录
        :param factor_id:因子ID
        :param instrument_id:合约编号
        :param db_client: Mongo客户端
        """
        factor_record_db = db_client.min_factor
        factor_record_collection = factor_record_db[str(factor_id)]
        factor_record_list = factor_record_collection.find({'instrument_id': str(instrument_id)})
        for factor_record in factor_record_list:
            df_asset = pd.DataFrame(factor_record['asset'])
            df_trade = pd.DataFrame(factor_record['trade'])
            df_summary = pd.DataFrame([factor_record['summary']])
            df_param = pd.DataFrame([factor_record['param']]).T
            account_id = factor_record['account_id']

            try:
                excel_record_path = '{0}/{1}/{2}.xlsx'.format(target_dir, factor_id, account_id)
                if not os.path.exists(target_dir):
                    os.makedirs(target_dir)

                dic_data = {
                    'Asset': df_asset,
                    'Trade': df_trade,
                    'Summary': df_summary,
                    'Param': df_param,
                }
                FileToolBox.save_excel(excel_record_path, **dic_data)
                fig_record_path = os.path.join(target_dir, factor_id, '{0}.png'.format(account_id))
                df_asset['累计收益率(%)'] = df_asset['累计收益率'].str.strip('%').astype(float) / 100
                FileToolBox.save_fig(fig_record_path, df_asset, '日期', ['累计收益率(%)'])
            except Exception as e:
                print(str(e))

    @staticmethod
    def export_factor_record_by_account_id(db_client, target_dir, factor_id, account_id):
        """
        根据因子ID、合约编号导出指定的数据集合
        :param db_client:
        :param account_id:
        :param target_dir:导出目录
        :param factor_id:因子ID
        """
        factor_record_db = db_client.min_factor
        factor_record_collection = factor_record_db[str(factor_id)]
        factor_record_list = factor_record_collection.find({'account_id': str(account_id)})
        for factor_record in factor_record_list:
            df_asset = pd.DataFrame(factor_record['asset'])
            df_trade = pd.DataFrame(factor_record['trade'])
            df_summary = pd.DataFrame([factor_record['summary']])
            df_param = pd.DataFrame([factor_record['param']]).T
            account_id = factor_record['account_id']

            try:
                excel_record_path = '{0}/{1}/{2}.xlsx'.format(target_dir, factor_id, account_id)
                if not os.path.exists(target_dir):
                    os.makedirs(target_dir)

                dic_data = {
                    'Asset': df_asset,
                    'Trade': df_trade,
                    'Summary': df_summary,
                    'Param': df_param,
                }
                FileToolBox.save_excel(excel_record_path, **dic_data)
                fig_record_path = os.path.join(target_dir, factor_id, '{0}.png'.format(account_id))
                df_asset['累计收益率(%)'] = df_asset['累计收益率'].str.strip('%').astype(float) / 100
                FileToolBox.save_fig(fig_record_path, df_asset, '日期', ['累计收益率(%)'])
            except Exception as e:
                print(str(e))

    @staticmethod
    def export_factor_record_net_fig(db_client, target_dir, factor_id, instrument_id):
        """
        根据因子ID、合约编号导出指定的净值走势
        :param db_client:
        :param target_dir:导出目录
        :param factor_id:因子ID
        :param instrument_id:合约编号
        """
        factor_record_db = db_client.min_factor
        factor_record_collection = factor_record_db[str(factor_id)]
        factor_record_list = factor_record_collection.find({'instrument_id': str(instrument_id)})
        for factor_record in factor_record_list:
            df_asset = pd.DataFrame(factor_record['asset'])
            account_id = factor_record['account_id']
            try:
                fig_record_path = os.path.join(target_dir, '{0}.png'.format(account_id))
                df_asset['累计收益率(%)'] = df_asset['累计收益率'].str.strip('%').astype(float) / 100
                FileToolBox.save_fig(fig_record_path, df_asset, '日期', ['累计收益率(%)'])
            except Exception as e:
                print(str(e))

    @staticmethod
    def export_factor_record_net_fig_by_account_id(db_client, target_dir, factor_id, account_id):
        """
        根据因子ID、合约编号导出指定的净值走势
        :param db_client:
        :param factor_id:
        :param target_dir:导出目录
        :param account_id:
        """
        factor_record_db = db_client.min_factor
        factor_record_collection = factor_record_db[str(factor_id)]
        factor_record_list = factor_record_collection.find({'account_id': str(account_id)})
        for factor_record in factor_record_list:
            df_asset = pd.DataFrame(factor_record['asset'])
            account_id = factor_record['account_id']
            try:
                fig_record_path = os.path.join(target_dir, '{0}.png'.format(account_id))
                df_asset['累计收益率(%)'] = df_asset['累计收益率'].str.strip('%').astype(float) / 100
                FileToolBox.save_fig(fig_record_path, df_asset, '日期', ['累计收益率(%)'])
            except Exception as e:
                print(str(e))

    @staticmethod
    def get_summary_year_by_account_id(db_client, account_id):
        """
        根据因子ID、合约编号导出指定的数据集合
        :param db_client:
        :param account_id:
        """
        factor_filter_db = db_client.summary_by_year
        factor_filter_collection = factor_filter_db['min']
        factor_filter_list = factor_filter_collection.find({'account_id': account_id})
        df_all_summary = None
        for factor_filter in factor_filter_list:
            df_summary = pd.DataFrame([factor_filter['summary']])
            if df_all_summary is None:
                df_all_summary = df_summary
            else:
                df_all_summary = df_all_summary.append(df_summary)

        return df_all_summary

    @staticmethod
    def export_factor_filter_record(db_client, target_dir):
        """
        根据因子ID、合约编号导出指定的数据集合
        :param db_client:
        :param target_dir:导出目录
        """
        factor_filter_db = db_client.summary_filter
        factor_filter_collection = factor_filter_db['min']
        factor_filter_list = factor_filter_collection.find()
        df_all_summary = None
        df_all_summary_by_year = None
        for factor_filter in factor_filter_list:
            df_summary = pd.DataFrame([factor_filter['summary']])
            if df_all_summary is None:
                df_all_summary = df_summary
            else:
                df_all_summary = df_all_summary.append(df_summary)
            factor_id = factor_filter['summary']['FactorId']
            account_id = factor_filter['account_id']

            MongoHelper.export_factor_record_by_account_id(db_client, os.path.join(target_dir, 'MinFactor'), factor_id, account_id)
            df_summary_by_year = MongoHelper.get_summary_year_by_account_id(db_client, account_id)
            if df_all_summary_by_year is None:
                df_all_summary_by_year = df_summary_by_year
            else:
                df_all_summary_by_year = df_all_summary_by_year.append(df_summary_by_year)
        FileToolBox.save_csv(df_all_summary, os.path.join(target_dir, 'AllSummary.csv'))
        FileToolBox.save_csv(df_all_summary_by_year, os.path.join(target_dir, 'AllSummaryByYear.csv'))

# db_host = 'mongodb://admin:123456@192.168.1.202:27017/'
# export_dir = 'D:/MongodbRecord'
# client = MongoClient(db_host)
# MongoHelper.export_factor_filter_record(os.path.join(export_dir, 'FactorFilter'))
# MongoHelper.export_factor_record_by_instrument_id(os.path.join(export_dir, 'MinFactor'))




import datetime
import os
import sys


class TradingDayHelper:
    """
    交易日帮助类
    """
    _days = None
    IsLoaded = False

    @staticmethod
    def trading_days():
        """
        交易日
        :return:
        """
        if TradingDayHelper._days is None:
            TradingDayHelper.get_all_china_trading_days()
        return TradingDayHelper._days

    @staticmethod
    def get_all_china_trading_days():
        """
        获取中国交易日列表
        """
        TradingDayHelper._days = []
        trading_calendar_dir = os.path.join(sys.modules['ROOT_DIR'], 'Profiles', 'TradingCalendar')
        trading_calendar_files = os.listdir(trading_calendar_dir)  # 列出文件夹下所有的目录与文件
        for i in range(0, len(trading_calendar_files)):
            trading_day_file = os.path.join(trading_calendar_dir, trading_calendar_files[i])
            if os.path.isfile(trading_day_file):
                for line in open(trading_day_file, "r"):  # 设置文件对象并读取每一行文件
                    n_trading_date = int(line)
                    year = n_trading_date // 10000
                    month = n_trading_date % 10000 // 100
                    day = n_trading_date % 100
                    trading_date = datetime.datetime(year, month, day)
                    TradingDayHelper._days.append(trading_date)
        TradingDayHelper.IsLoaded = True

    @staticmethod
    def get_trading_days(begin_date, end_date):
        """
        获取日期区间内的所有交易日
        :param begin_date:
        :param end_date:
        :return:
        """
        list_get_trading_day = []
        begin_date = datetime.datetime(begin_date.year, begin_date.month, begin_date.day)
        end_date = datetime.datetime(end_date.year, end_date.month, end_date.day)
        trading_days = TradingDayHelper.trading_days()
        for trading_day in trading_days:
            if (trading_day >= begin_date) & (trading_day <= end_date):
                list_get_trading_day.append(trading_day)
        return list_get_trading_day

    @staticmethod
    def get_natural_date_time(trading_date, time_span):
        """
        根据交易日和时间切片生成自然日时间
        :param trading_date:
        :param time_span:
        :return:
        """
        natural_time = trading_date + time_span
        hours = natural_time.hour
        if hours <= 6:
            natural_time = TradingDayHelper.get_pre_trading_day(trading_date) + datetime.timedelta(days=1) + time_span
        elif hours >= 16:
            natural_time = TradingDayHelper.get_pre_trading_day(trading_date) + time_span
        return natural_time

    @staticmethod
    def get_next_trading_day(date, n=1):
        """
        往后回溯n个交易日
        :param date:
        :param n:
        :return:
        """
        date = datetime.datetime(date.year, date.month, date.day)
        trading_days = TradingDayHelper.trading_days()
        index = -1
        if date in trading_days:
            index = trading_days.index(date)
        if index > -1:
            index_next = index + n
            index_next = min(len(trading_days) - 1, index_next)
            return trading_days[index_next]
        return TradingDayHelper.get_first_trading_day(date)

    @staticmethod
    def get_pre_trading_day(date, n=1):
        """
        往前回溯n个交易日
        :param date:交易日
        :param n:回溯天数
        :return:
        """
        date = datetime.datetime(date.year, date.month, date.day)
        trading_days = TradingDayHelper.trading_days()
        if date in trading_days:
            index = trading_days.index(date)
        else:
            date = TradingDayHelper.get_last_trading_day(date)
            index = trading_days.index(date)
            index = index + 1  # 非交易日的情况 默认加一天
        if index > n:
            index_next = index - n
            index_next = max(0, index_next)
            return trading_days[index_next]
        return TradingDayHelper.get_last_trading_day(date)

    @staticmethod
    def get_first_trading_day(date):
        """
        获取大于等于日期的第一个交易日
        :param date:
        :return:
        """
        date = datetime.datetime(date.year, date.month, date.day)
        for i in range(0, 20):
            if TradingDayHelper.is_china_trading_day(date):
                return date

            date = date + datetime.timedelta(days=1)

    @staticmethod
    def get_last_trading_day(date):
        """
        获取小于等于日期的最后一个交易日
        :param date:
        :return:
        """
        date = datetime.datetime(date.year, date.month, date.day)
        for i in range(0, 20):
            if TradingDayHelper.is_china_trading_day(date):
                return date

            date = date + datetime.timedelta(days=-1)

    @staticmethod
    def get_trading_day_count(begin_date, end_date):
        """
        获取交易区间的交易日个数，是闭区间，包含头尾的
        :param begin_date:
        :param end_date:
        :return:
        """
        trading_days = TradingDayHelper.trading_days()
        begin_date = TradingDayHelper.get_first_trading_day(begin_date)
        end_date = TradingDayHelper.get_last_trading_day(end_date)
        begin_index = trading_days.index(begin_date)
        end_index = trading_days.index(end_date)
        count = 0
        if begin_index <= end_index:
            count = end_index - begin_index + 1
        return count

    @staticmethod
    def is_china_trading_day(date):
        """
        是否为中国交易日
        :param date:
        :return:
        """
        date = datetime.datetime(date.year, date.month, date.day)
        trading_days = TradingDayHelper.trading_days()
        if date not in trading_days:
            return False
        return True

# begin_date_test = datetime.datetime(2019, 4, 1)
# end_date_test = datetime.datetime(2019, 10, 1)
# next_trading_day = TradingDayHelper.get_next_trading_day(end_date_test)
# pre_trading_day = TradingDayHelper.get_pre_trading_day(end_date_test)
# first_trading_day = TradingDayHelper.get_first_trading_day(end_date_test)
# last_trading_day = TradingDayHelper.get_last_trading_day(end_date_test)
# trading_day_count = TradingDayHelper.get_trading_day_count(begin_date_test, end_date_test)
# print(next_trading_day.strftime('%Y/%m/%d'))
# print("pre_trading_day"+pre_trading_day.strftime('%Y/%m/%d'))
# print(first_trading_day.strftime('%Y/%m/%d'))
# print(last_trading_day.strftime('%Y/%m/%d'))
# print(str(trading_day_count))
# for a in data:
#     print(a.strftime('%Y/%m/%d'))



"""
Description:
    Binary Reader
Usage:
    from neo.core.IO.BinaryReader import BinaryReader
"""
import sys
import struct
import binascii
import importlib


class BinaryReader(object):
    """docstring for BinaryReader"""

    def __init__(self, stream):
        """
        Create an instance.
        Args:
            stream (BytesIO): a stream to operate on. i.e. a neo.IO.MemoryStream or raw BytesIO.
        """
        super(BinaryReader, self).__init__()
        self.stream = stream

    def unpack(self, fmt, length=1):
        """
        Unpack the stream contents according to the specified format in `fmt`.
        For more information about the `fmt` format see: https://docs.python.org/3/library/struct.html
        Args:
            fmt (str): format string.
            length (int): amount of bytes to read.
        Returns:
            variable: the result according to the specified format.
        """
        return struct.unpack(fmt, self.stream.read(length))[0]

    def read_byte(self, do_ord=True):
        """
        read a single byte.
        Args:
            do_ord (bool): (default True) convert the byte to an ordinal first.
        Returns:
            bytes: a single byte if successful. 0 (int) if an exception occurred.
        """
        try:
            if do_ord:
                return ord(self.stream.read(1))
            return self.stream.read(1)
        except Exception as e:
            print("ord expected character but got none")
        return 0

    def read_bytes(self, length):
        """
        read the specified number of bytes from the stream.
        Args:
            length (int): number of bytes to read.
        Returns:
            bytes: `length` number of bytes.
        """
        value = self.stream.read(length)
        return value

    def safe_read_bytes(self, length):
        """
        read exactly `length` number of bytes from the stream.
        Raises:
            ValueError is not enough data
        Returns:
            bytes: `length` number of bytes
        """
        data = self.read_bytes(length)
        if len(data) < length:
            raise ValueError("Not enough data available")
        else:
            return data

    def read_bool(self):
        """
        read 1 byte as a boolean value from the stream.
        Returns:
            bool:
        """
        return self.unpack('?')

    def read_char(self):
        """
        read 1 byte as a character from the stream.
        Returns:
            str: a single character.
        """
        return self.unpack('c')

    def read_float(self, endian="<"):
        """
        read 4 bytes as a float value from the stream.
        Args:
            endian (str): specify the endianness. (Default) Little endian ('<'). Use '>' for big endian.
        Returns:
            float:
        """
        return self.unpack("%sf" % endian, 4)

    def read_double(self, endian="<"):
        """
        read 8 bytes as a double value from the stream.
        Args:
            endian (str): specify the endianness. (Default) Little endian ('<'). Use '>' for big endian.
        Returns:
            float:
        """
        return self.unpack("%sd" % endian, 8)

    def read_int8(self, endian="<"):
        """
        read 1 byte as a signed integer value from the stream.
        Args:
            endian (str): specify the endianness. (Default) Little endian ('<'). Use '>' for big endian.
        Returns:
            int:
        """
        return self.unpack('%sb' % endian)

    def read_u_int8(self, endian="<"):
        """
        read 1 byte as an unsigned integer value from the stream.
        Args:
            endian (str): specify the endianness. (Default) Little endian ('<'). Use '>' for big endian.
        Returns:
            int:
        """
        return self.unpack('%sB' % endian)

    def read_int16(self, endian="<"):
        """
        read 2 byte as a signed integer value from the stream.
        Args:
            endian (str): specify the endianness. (Default) Little endian ('<'). Use '>' for big endian.
        Returns:
            int:
        """
        return self.unpack('%sh' % endian, 2)

    def read_u_int16(self, endian="<"):
        """
        read 2 byte as an unsigned integer value from the stream.
        Args:
            endian (str): specify the endianness. (Default) Little endian ('<'). Use '>' for big endian.
        Returns:
            int:
        """
        return self.unpack('%sH' % endian, 2)

    def read_int32(self, endian="<"):
        """
        read 4 bytes as a signed integer value from the stream.
        Args:
            endian (str): specify the endianness. (Default) Little endian ('<'). Use '>' for big endian.
        Returns:
            int:
        """
        return self.unpack('%si' % endian, 4)

    def read_u_int32(self, endian="<"):
        """
        read 4 bytes as an unsigned integer value from the stream.
        Args:
            endian (str): specify the endianness. (Default) Little endian ('<'). Use '>' for big endian.
        Returns:
            int:
        """
        return self.unpack('%sI' % endian, 4)

    def read_int64(self, endian="<"):
        """
        read 8 bytes as a signed integer value from the stream.
        Args:
            endian (str): specify the endianness. (Default) Little endian ('<'). Use '>' for big endian.
        Returns:
            int:
        """
        return self.unpack('%sq' % endian, 8)

    def read_u_int64(self, endian="<"):
        """
        read 8 bytes as an unsigned integer value from the stream.
        Args:
            endian (str): specify the endianness. (Default) Little endian ('<'). Use '>' for big endian.
        Returns:
            int:
        """
        return self.unpack('%sQ' % endian, 8)

    def read_var_int(self, max=sys.maxsize):
        """
        read a variable length integer from the stream.
        The NEO network protocol supports encoded storage for space saving. See: http://docs.neo.org/en-us/node/network-protocol.html#convention
        Args:
            max (int): (Optional) maximum number of bytes to read.
        Returns:
            int:
        """
        fb = self.read_byte()
        if fb == 0:
            return fb
        value = 0
        if hex(fb) == '0xfd':
            value = self.read_u_int16()
        elif hex(fb) == '0xfe':
            value = self.read_u_int32()
        elif hex(fb) == '0xff':
            value = self.read_u_int64()
        else:
            value = fb

        if value > max:
            raise Exception("Invalid format")

        return int(value)

    def read_var_bytes(self, max=sys.maxsize):
        """
        read a variable length of bytes from the stream.
        The NEO network protocol supports encoded storage for space saving. See: http://docs.neo.org/en-us/node/network-protocol.html#convention
        Args:
            max (int): (Optional) maximum number of bytes to read.
        Returns:
            bytes:
        """
        length = self.read_var_int(max)
        return self.read_bytes(length)

    def read_string(self):
        """
        read a string from the stream.
        Returns:
            str:
        """
        length = self.read_u_int8()
        return self.unpack(str(length) + 's', length)

    def read_var_string(self, max=sys.maxsize):
        """
        Similar to `read_string` but expects a variable length indicator instead of the fixed 1 byte indicator.
        Args:
            max (int): (Optional) maximum number of bytes to read.
        Returns:
            bytes:
        """
        length = self.read_var_int(max)
        return self.unpack(str(length) + 's', length)

    def read_fixed_string(self, length):
        """
        read a fixed length string from the stream.
        Args:
            length (int): length of string to read.
        Returns:
            bytes:
        """
        return self.read_bytes(length).rstrip(b'\x00')

    def read_serializable_array(self, class_name, max=sys.maxsize):
        """
        Deserialize a stream into the object specific by `class_name`.
        Args:
            class_name (str): a full path to the class to be deserialized into. e.g. 'neo.Core.Block.Block'
            max (int): (Optional) maximum number of bytes to read.
        Returns:
            list: list of `class_name` objects deserialized from the stream.
        """
        module = '.'.join(class_name.split('.')[:-1])
        klassname = class_name.split('.')[-1]
        klass = getattr(importlib.import_module(module), klassname)
        length = self.read_var_int(max=max)
        items = []
        #        logger.info("READING ITEM %s %s " % (length, class_name))
        try:
            for i in range(0, length):
                item = klass()
                item.Deserialize(self)
                #                logger.info("deserialized item %s %s " % ( i, item))
                items.append(item)
        except Exception as e:
            print("Couldn't deserialize %s " % e)

        return items

    def read2000256_list(self):
        """
        read 2000 times a 64 byte value from the stream.
        Returns:
            list: a list containing 2000 64 byte values in reversed form.
        """
        items = []
        for i in range(0, 2000):
            data = self.read_bytes(64)
            ba = bytearray(binascii.unhexlify(data))
            ba.reverse()
            items.append(ba.hex().encode('utf-8'))
        return items

    def read_hashes(self):
        """
        read Hash values from the stream.
        Returns:
            list: a list of hash values. Each value is of the bytearray type.
        """
        len = self.read_var_int()
        items = []
        for i in range(0, len):
            ba = bytearray(self.read_bytes(32))
            ba.reverse()
            items.append(ba.hex())
        return items



from .OldStrategy import OldStrategy
from .BaseStrategy import BaseStrategy
from .StrategyToolBox import StrategyToolBox
from .Function import Function
from .MixStrategy import MixStrategy
from .SimulateExecutor import SimulateExecutor
from .StrategyEngine import StrategyEngine

__all__ = [
    'OldStrategy',
    'BaseStrategy',
    'StrategyToolBox',
    'Function',
    'MixStrategy',
    'SimulateExecutor',
    'StrategyEngine'
]



class LimitValue:
    """
    涨跌停限制
    """
    def __init__(self):
        self._up_limit = 0.1
        self._low_limit = 0.1

    @property
    def up_limit(self):
        """

        :return:
        """
        return self._up_limit

    @up_limit.setter
    def up_limit(self, value):
        self._up_limit = value

    @property
    def low_limit(self):
        """

        :return:
        """
        return self._low_limit

    @low_limit.setter
    def low_limit(self, value):
        self._low_limit = value



from abc import ABC

from SapphireQuant.Core.Enum import EnumBuySell
from SapphireQuant.Simulate.Match.BaseMatchMachine import BaseMatchMachine


class LastPriceMatchMachine(BaseMatchMachine, ABC):
    """
    最新价撮合机
    """
    def __init__(self):
        super().__init__()

    def cancel(self, order):
        """
        撤单
        :param order:
        :return:
        """
        if order.instrument_id in self.order_map.keys():
            orders = self.order_map[order.instrument_id]
            for i in range(len(orders)):
                data = orders[i]
                if data.local_id == order.loacl_id or data.order_sys_id == order.order_sys_id:
                    order = data
                    del orders[i]
                    break
            msg = '撤单成功'
            return True, order, msg

        msg = '未找到委托编号为{0}的订单'.format(order.order_sys_id)
        return False, order, msg

    def can_match_by_bar(self, bar, order):
        """

        :param bar:
        :param order:
        :return:
        """
        if order.direction == EnumBuySell.买入:
            if order.order_price >= bar.close:
                match_price = bar.close
                match_volume = order.order_volume
                return True, match_price, match_volume

        elif order.direction == EnumBuySell.卖出:
            if order.order_price <= bar.close:
                match_price = bar.close
                match_volume = order.order_volume
                return True, match_price, match_volume

        return False, None, None

    def can_match_by_tick(self, tick, order):
        """

        :param tick:
        :param order:
        :return:
        """
        if order.direction == EnumBuySell.买入:
            if order.order_price > tick.bid_price1:
                match_price = min(order.order_price, tick.last_price)
                match_volume = order.order_volume
                return True, match_price, match_volume

        elif order.direction == EnumBuySell.卖出:
            if order.order_price < tick.ask_price1:
                match_price = max(order.order_price, tick.last_price)
                match_volume = order.order_volume
                return True, match_price, match_volume

        return False, None, None


from .Match import *

from .DataBlock import DataBlock
from .ReplayData import ReplayData
from .SimulateFutureQuoteChannel import SimulateFutureQuoteChannel

__all__ = [
    'DataBlock',
    'ReplayData',
    'SimulateFutureQuoteChannel',
]
__all__.extend(Match.__all__)